/*
 *  A libESMTP Example Application.
 *  Copyright (C) 2001,2002  Brian Stafford <brian@stafford.uklinux.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2 of the License,
 *  or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* This program accepts a single file argument followed by a list of
   recipients.  The file is mailed to each of the recipients.

   Error checking is minimal to non-existent, this is just a quick
   and dirty program to give a feel for using libESMTP.
 */
#define _XOPEN_SOURCE 500

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <stdarg.h>
#include <syslog.h>

#include <openssl/ssl.h>
#include <auth-client.h>
#include <libesmtp.h>

#if !defined (__GNUC__) || __GNUC__ < 2
# define __attribute__(x)
#endif
#define unused      __attribute__((unused))

/* SMTP authentication'i sirasinda kullanilan username ve password bu dosyadan okunuyor. */
#define USER_PASSWD_FILE "/etc/smtpauth" 

//#define DEBUG

#define SEPARATOR (char)1 /* Mesaj'in header ve data kisimlarinin sonuna \r\n koymak icin kullaniliyor
			     Ayrintinlar readline_cb_memory function'i icinde */


const char * readline_cb_memory(void **buf, int *len, void *message_text);
int simple_send_mail(char *MTAHost, char *from, char *to, char *subject, char *mesaj);
int authinteract(auth_client_request_t request, char **result, int fields, void *arg unused);
void monitor_cb(const char *buf, int buflen, int writing, void *arg);
void event_cb(smtp_session_t session, int event_no, void *arg,...);
int tlsinteract(char *buf, int buflen, int rwflag unused, void *arg unused);
int handle_invalid_peer_certificate(long vfy_result);




/* returns status->code (250 when succefful) or 0 when an error occurs */
int simple_send_mail(char *MTAHost, char *from, char *to, char *subject, char *mesaj)
{
	smtp_session_t session;
	smtp_message_t message;
	smtp_recipient_t recipient;
	auth_context_t authctx;
	const smtp_status_t *status;
	struct sigaction sa;
	char buf[4096];
	char *mtaHost=(char *)malloc( strlen(MTAHost)+4 );
	//enum notify_flags notify = Notify_NOTSET;
	/*notify |= Notify_SUCCESS | Notify_FAILURE | Notify_DELAY;*/
  
	session = smtp_create_session();
	message = smtp_add_message(session);

	smtp_starttls_enable(session, Starttls_ENABLED);

	#ifdef DEBUG
	    smtp_set_monitorcb(session, monitor_cb, stdout, 1);
	#endif

   /* NB.  libESMTP sets timeouts as it progresses through the protocol.
     In addition the remote server might close its socket on a timeout.
     Consequently libESMTP may sometimes try to write to a socket with
     no reader.  Ignore SIGPIPE, then the program doesn't get killed
     if/when this happens. */
	sa.sa_handler = SIG_IGN;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	sigaction(SIGPIPE, &sa, NULL); 
	
	strcpy(mtaHost, MTAHost);
	strcat(mtaHost, ":25");
	if( !smtp_set_server(session, mtaHost) ) {
		syslog(LOG_INFO, "smptp_set_server: error !");
		free(mtaHost); return 0;
	}
	free(mtaHost);
	

	/* Do what's needed at application level to use authentication. */
	authctx = auth_create_context();
	auth_set_mechanism_flags(authctx, AUTH_PLUGIN_PLAIN, 0);
	auth_set_interact_cb(authctx, authinteract, NULL);


	/* Use our callback for X.509 certificate passwords.  If STARTTLS is
	   not in use or disabled in configure, the following is harmless. */
	smtp_starttls_set_password_cb(tlsinteract, NULL);
	smtp_set_eventcb(session, event_cb, NULL);

	
	/* Now tell libESMTP it can use the SMTP AUTH extension. */
	smtp_auth_set_context(session, authctx);

	

	smtp_set_reverse_path(message, from);
	recipient = smtp_add_recipient(message, to);
	
	//if( notify != Notify_NOTSET ) smtp_dsn_set_notify(recipient, notify);

	smtp_set_header_option(message, "Date", Hdr_OVERRIDE, 1);

	sprintf(buf,"Date:X%cMessage-Id:www.basel.name.tr%cFrom:%s%cSubject:%s%cTo:%s%c%c%s%c", SEPARATOR, SEPARATOR, from, SEPARATOR, subject,SEPARATOR, to, SEPARATOR, SEPARATOR, mesaj, SEPARATOR);

#ifdef DEBUG
        syslog(LOG_INFO, "mesaj buf:%s", buf);
#endif	
	/* Gonderilecek mesajin okunmasini saglayacak function "readline_cb_memory" oldugu bildiriliyor
	   bu function buf parametresi ile cagrilacak, gonderilecek data bitene kadar bu function tekrar tekrar cagiriliyor.
	   Bu function library icinde yer almiyor, asagida tanimli.
	   Bu function'in gorevi data'nin once header kismini gondermek, gonderilen her header satirinin sonuna \r\n koyuyor.
	   Header'lar bittikten sonra bos bir satir gonderiyor, bunun anlami bundan sonra mesajin baslayacagini bildirmek oluyor.
	   Mesaj da gonderiliyor, bitince yine \r\n gonderiliyor.
	   Tum veri bitince NULL gonderiliyor, bundan sonra library mesjin bittigini anliyor ve mesji sonlandiriyor.
	   Bu function'a gonderilen buf icinde header'lar ve mesaj SEPARATOR karakteri ile ayrilmis durumda.
	   Function'in yaptigi is SEPARATOR karakteri yerine \r\n koymak oluyor.
	*/

	smtp_set_messagecb(message, readline_cb_memory, &buf);
		
	/* Initiate a connection to the SMTP server and transfer the message. */
	if( !smtp_start_session(session) ) {
		syslog(LOG_ERR, "SMTP server problem: %s\n", smtp_strerror(smtp_errno(), buf, sizeof buf));
		/* Free resources consumed by the program. */
		smtp_destroy_session (session);
		return 0;
	}

	/* Report on the success or otherwise of the mail transfer. */
	status = smtp_message_transfer_status(message);
#ifdef DEBUG
	syslog(LOG_INFO, "smtp transfer status: %d", status->code);
#endif
	
	/* Free resources consumed by the program. */
	smtp_destroy_session (session);
	return status->code;
}


/* Callback to request user/password info.  Not thread safe. */
int authinteract(auth_client_request_t request, char **result, int fields, void *arg unused)
{
	static char username[66], password[66];
	FILE *auth;
	
	#ifdef DEBUG
		syslog(LOG_INFO, "authinteract function has been called\n");
	#endif 
	
	auth=fopen(USER_PASSWD_FILE, "r");
	if( !auth ) {
		syslog(LOG_ERR, "Can't find file USER_PASSWD_FILE:%s\n", USER_PASSWD_FILE);
		return 1;
	}

	
	/* Mail server'a SMTP ile mail gonderirken authentication gerekiyorsa
	   fields=1, request[0].prompt'da user name prompt'u ve request[1].prompt'da da passwd prompt'u geliyor
	   Cevap olarak verilecek bilgi string'lere atanacak ve bunlarin adresleri result degiskenine konur
	   string'ler static ile tanimlanan kalici hafizada tutuluyor, bu function'dan cikinca yok olmamalari icin
	*/		

	fgets(username, 64, auth); username[ strlen(username)-1 ]=(char)NULL; result[0]=username;
    
	fgets(password, 64, auth); password[ strlen(password)-1 ]=(char)NULL; result[1]=password;

	fclose( auth );
	
	#ifdef DEBUG
		syslog(LOG_INFO, "Username:%s, password:%s.\n", result[0], result[1]);
	#endif 
	return 1;
}



 /* Bu function ornek programi kullanilarak yazilmistir; ornek program mesaji bir dosyadan okuyordu
    bu function ise mesaji memory'den kendisine gonderilen message_text parametresinden aliyor.
    Bu function'a gelen data icinde SEPARATOR karakteri yerine \r\n koyarak iki SEPARATOR arasindaki kisim 
    malloc ile alinan bir hafizaya konuluyor ve bu adres buf parametresi ile library'ye gonderiliyor,
    library bu kismi karsi taraftaki mail server'a gonderip hafiza'yi free yapiyor, sonra tekrar bu function'i cagiriyor.
    Tum data bittiginde len=0 yapilip NULL geri gonderiliyor, bu durumda library data'nin bittigini anlayip mesaji sonlandiriyor.
 */
const char *readline_cb_memory(void **buf, int *len, void *message_text)
{
	static unsigned int yer=0; /* will hold the current location in the string */
	char *mes=NULL;
	char *pstart;
	char *ps;
	int unsigned total_length=strlen(message_text);
  
	/* Copy the entire to a newly allocated memory, to be able to make some character changes
	like replacing SEPARATOR with NULL and inserting \r\n characters */
	mes=(char *)malloc( total_length+13 );
	if( !mes ) {
		syslog(LOG_ERR, "readline_cb_memory: Couldn't allocate memory !");
		return NULL;
	}
	strcpy(mes, message_text);
  
	/* printf("Total Message text: %s-\n", mes); */
  
	if( !len ) {
		/* puts("len=NULL, return"); */
		yer=0; free(mes); return NULL;
	}

	/* printf("Gelen len=%d, kalan mes[%d]:%s-\n", *len, yer, &mes[yer]); */
	
	if( yer>=total_length ) {
		/* puts("Mesaj bitti."); */
		*len=0; *buf=NULL;  	
		free(mes); return NULL;
    }
	
	pstart=mes; /* pstart holds the beginning address of the whole message string */
	ps=strchr(&mes[yer], SEPARATOR); ps[0]=(char)NULL; /* finish the string mes */
	ps++; /* ps will now point the next character after '\0' */
	/*printf("gonderilecek veri:%s-, sonraki kisim:%s, yer=%d\n", &mes[yer], ps, yer);*/
					
	strcpy(ps-1, "\r\n");
		
	*len = strlen(&mes[yer]);
	*buf=(char *)malloc( *len+4 );
	if( !*buf ) {
		syslog(LOG_ERR, "readline_cb_memory: Couldn't allocate memory !");
		free(mes); *len=0; return NULL;
	}
	strcpy( *buf, &mes[yer] );
    	yer=(int)(ps-pstart); // find the length of the first part of the string using pointer addresses
	
	/*printf("yeni len=%d, buf=%s--\n", *len, (char *)*buf);*/
	free(mes); 
	return *buf;
}


void monitor_cb(const char *buf, int buflen, int writing, void *arg)
{
  FILE *fp = arg;

  if (writing == SMTP_CB_HEADERS)
    {
      fputs ("H: ", fp);
      fwrite (buf, 1, buflen, fp);
      return;
    }

 fputs (writing ? "C: " : "S: ", fp);
 fwrite (buf, 1, buflen, fp);
 if (buf[buflen - 1] != '\n')
   putc ('\n', fp);
}


void event_cb(smtp_session_t session, int event_no, void *arg,...)
{
  va_list alist;
  int *ok;
  
  va_start(alist, arg);
  switch(event_no) {
  case SMTP_EV_CONNECT: 
  case SMTP_EV_MAILSTATUS:
  case SMTP_EV_RCPTSTATUS:
  case SMTP_EV_MESSAGEDATA:
  case SMTP_EV_MESSAGESENT:
  case SMTP_EV_DISCONNECT: break;
  case SMTP_EV_WEAK_CIPHER: {
    int bits;
    bits = va_arg(alist, long); ok = va_arg(alist, int*);
    #ifdef DEBUG
        syslog(LOG_INFO, "SMTP_EV_WEAK_CIPHER, bits=%d - accepted.\n", bits);
    #endif
    *ok = 1; break;
  } 
  case SMTP_EV_STARTTLS_OK:
    #ifdef DEBUG
       syslog(LOG_INFO,"SMTP_EV_STARTTLS_OK - TLS started here.");
    #endif
    break;
  case SMTP_EV_INVALID_PEER_CERTIFICATE: {
    long vfy_result;
    vfy_result = va_arg(alist, long); ok = va_arg(alist, int*);
    *ok = handle_invalid_peer_certificate(vfy_result);
    break;
  }
  case SMTP_EV_NO_PEER_CERTIFICATE: {
    ok = va_arg(alist, int*); 
    #ifdef DEBUG
        syslog(LOG_INFO, "SMTP_EV_NO_PEER_CERTIFICATE - accepted.");
    #endif
    *ok = 1; break;
  }
  case SMTP_EV_WRONG_PEER_CERTIFICATE: {
    ok = va_arg(alist, int*);
    #ifdef DEBUG
        syslog(LOG_INFO, "SMTP_EV_WRONG_PEER_CERTIFICATE - accepted.");
    #endif
    *ok = 1; break;
  }
  case SMTP_EV_NO_CLIENT_CERTIFICATE: {
    ok = va_arg(alist, int*);
    #ifdef DEBUG
        syslog(LOG_INFO, "SMTP_EV_NO_CLIENT_CERTIFICATE - accepted.");
    #endif
    *ok = 1; break;
  }
  default:
        syslog(LOG_INFO, "Got event: %d - ignored.\n", event_no);
  }
  va_end(alist);
}


int tlsinteract(char *buf, int buflen, int rwflag unused, void *arg unused)
{
  char *pw;
  int len;

  pw = getpass ("certificate password");
  len = strlen (pw);
  if (len + 1 > buflen)
    return 0;
  strcpy (buf, pw);
  return len;
}

int handle_invalid_peer_certificate(long vfy_result)
{
  const char *k ="rare error";
  switch(vfy_result) {
  case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
    k="X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT"; break;
  case X509_V_ERR_UNABLE_TO_GET_CRL:
    k="X509_V_ERR_UNABLE_TO_GET_CRL"; break;
  case X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE:
    k="X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE"; break;
  case X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE:
    k="X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE"; break;
  case X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY:
    k="X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY"; break;
  case X509_V_ERR_CERT_SIGNATURE_FAILURE:
    k="X509_V_ERR_CERT_SIGNATURE_FAILURE"; break;
  case X509_V_ERR_CRL_SIGNATURE_FAILURE:
    k="X509_V_ERR_CRL_SIGNATURE_FAILURE"; break;
  case X509_V_ERR_CERT_NOT_YET_VALID:
    k="X509_V_ERR_CERT_NOT_YET_VALID"; break;
  case X509_V_ERR_CERT_HAS_EXPIRED:
    k="X509_V_ERR_CERT_HAS_EXPIRED"; break;
  case X509_V_ERR_CRL_NOT_YET_VALID:
    k="X509_V_ERR_CRL_NOT_YET_VALID"; break;
  case X509_V_ERR_CRL_HAS_EXPIRED:
    k="X509_V_ERR_CRL_HAS_EXPIRED"; break;
  case X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD:
    k="X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD"; break;
  case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
    k="X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD"; break;
  case X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD:
    k="X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD"; break;
  case X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD:
    k="X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD"; break;
  case X509_V_ERR_OUT_OF_MEM:
    k="X509_V_ERR_OUT_OF_MEM"; break;
  case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
    k="X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT"; break;
  case X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN:
    k="X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN"; break;
  case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
    k="X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY"; break;
  case X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE:
    k="X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE"; break;
  case X509_V_ERR_CERT_CHAIN_TOO_LONG:
    k="X509_V_ERR_CERT_CHAIN_TOO_LONG"; break;
  case X509_V_ERR_CERT_REVOKED:
    k="X509_V_ERR_CERT_REVOKED"; break;
  case X509_V_ERR_INVALID_CA:
    k="X509_V_ERR_INVALID_CA"; break;
  case X509_V_ERR_PATH_LENGTH_EXCEEDED:
    k="X509_V_ERR_PATH_LENGTH_EXCEEDED"; break;
  case X509_V_ERR_INVALID_PURPOSE:
    k="X509_V_ERR_INVALID_PURPOSE"; break;
  case X509_V_ERR_CERT_UNTRUSTED:
    k="X509_V_ERR_CERT_UNTRUSTED"; break;
  case X509_V_ERR_CERT_REJECTED:
    k="X509_V_ERR_CERT_REJECTED"; break;
  }
  #ifdef DEBUG
     syslog(LOG_INFO, "SMTP_EV_INVALID_PEER_CERTIFICATE: %ld: %s\n", vfy_result, k);
  #endif
  return 1; /* Accept the problem */
}

