/* Network Tracker
Copyright Ali BASEL
Ali BASEL <alib@sabanciuniv.edu>, <ali@basel.name.tr>
http://www.basel.name.tr
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
*/
#include <ping.h>
#include <sys/types.h>
#include <unistd.h>
#include <syslog.h>
#include <signal.h>
#include <time.h>
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <netdb.h>
#include <sys/socket.h> /* for AF_INET */

#include "tracker.h"

#ifdef LINUX
struct option longopts[] = {
    { "help", no_argument, NULL, 'h', },
    { "version", no_argument, NULL, 'v', },
    { "n", no_argument, NULL, 'n', },
    { NULL, 0, NULL, 0, }
};
#endif

#define TIME1 8
#define TIME2 16

#undef DEMO_USE_SNMP_VERSION_3

#define PING_TIMEOUT 3
#define WAIT 60

#define TRACKERPIDFILE "/var/run/tracker.pid"

int sendmail(struct ayr *ayarlar);
int isle(struct ayr *ayarlar);
void usage(void);
void cik(int signo);
char *cevir(char *fin, char *address);
int snmp(char *host, struct ayr *ayarlar);


void usage(void)
{
	printf("\nVersion: %s\n", VERSION);
	puts("Usage: trackerd                    Run as a daemon");
	puts("       trackerd -n                 Don't run as a daemon");
	puts("       trackerd -h, --help         This info");
	puts("       trackerd -v, --version      This info");
	puts("\nPlease see man trackerd(1) for more info or read the README file.");
	puts("\nAli BASEL <alib@sabanciuniv.edu>\nhttp://www.basel.name.tr\n");
	exit(0);
}

int main(int argc, char **argv)
{
	int daemon=1, pid, op, wait_counter=0;
	int status=0, cevap_vermeyen_IP_sayisi=0, report=1, first_start=1;
	struct ayr ayarlar;
	time_t curtime, NewRunTime_timetick;
	struct tm *tmb, time_struct;
	char buf[MAXSTR];
	char year[5], month[3], day[3], hour[3], minute[3];
	FILE *trackerpidfile;
	char trackerpid_str[10];
	int trackerpid;



#ifdef LINUX  
	while( (op=getopt_long(argc, argv, "c::n::hv", longopts, NULL)) != EOF ) {
#else
	while( (op=getopt(argc, argv, "c::n::hv")) != EOF ) {
#endif
		switch(op) {
			case 'n':
				daemon = 0;
				break;
			default: usage();
		}
	}

	/* Read sql hostname, username and password from CONFIG_FILE file */
	if( !config_oku(&ayarlar) ) {
        	printf("Please look at syslog message file for the error !\n\n");
        	exit(0);
    	}
	
	if(daemon) {
		if( (pid=fork())<0) {
			syslog(LOG_ERR, "main fork() error %m");
			exit(1);
			/* for child process pid=0, for main process pid=number */
		}
		if( pid ) exit(0); /* main process ends, child is going to the background */
	
		/*
		No more output to the console is possible
		close( fileno(stdin) );
		close( fileno(stdout) );
		close( fileno(stderr) );	
		*/
		
	}

	/* When run with 'trackerd -n' check if another process is currently running */
	if( !daemon )
		if( Ayarlar(&ayarlar) && !strcmp(ayarlar.Status, "running") ) {
			printf("\nA tracking process is currently running !\nPlease wait until this process ends.\n\n");
			exit(1);
		}

	(void)signal(SIGINT, cik);
	(void)signal(SIGTERM, cik);
	(void)signal(SIGHUP, cik);
	(void)signal(SIGQUIT, cik);
	
	printf("Look at syslog (facility LOCAL1) for more info !\n");
	openlog("trackerd", LOG_NOWAIT|LOG_PID,  LOG_LOCAL1);
	syslog(LOG_INFO, "trackerd started");
	ayarlar.Period=0; // initial value	
	do {
		/* Read the settings from the database and fill the structure with these values
		 * Cevap vermeyen cihaz oldugunda gonderilecek mail ile 
		 * ilgili bilgiler aliniyor. */
		if( !Ayarlar(&ayarlar) ) {
			syslog(LOG_INFO, "main: Couldn't open the database, for reading the settings !");
			syslog(LOG_INFO, "main: sleeping 5 minutes, the connection will be checked again !");
			sprintf(buf, "Couldn't open the database for reading the settings !\nWill sleep 5 min., then the connection will be checked again.");
			simple_send_mail(ayarlar.MTAHost, ayarlar.FromName, ayarlar.To1, "trackerd Problem !", buf);
			sleep( 300 ); wait_counter+=300;
			continue;
		}

		if( first_start || wait_counter>=ayarlar.Period ) {
			wait_counter=0;
			syslog(LOG_INFO, "trackerd has started to check devices");
			status=IPListAlYaz(&ayarlar);
			if( !status ) {
				syslog(LOG_INFO, "main: Couldn't read the IP addresses !");
			}
			else {
				/* Database'den CEVAPYOK dosyasina alinmis olan IP'ler ping ve snmp ile
				 * kontrol edilecek; cevap vermeyen olursa e-mail ile bildirilecek. */
				if( isle(&ayarlar) ) cevap_vermeyen_IP_sayisi=sendmail( &ayarlar );
			}
		}	
		/* Belirli saatlerde programin problemsiz olarak calistigi bildirilecek. */
		time(&curtime); tmb=localtime(&curtime);
		if( (tmb->tm_hour==TIME1 || tmb->tm_hour==TIME2) && report ) {
			sprintf(buf, "trackerd is running. Total checked IP number: %d.\n"\
						"These IPs have been checked by snmp and ping.\n"\
						"Total Unreachable IP addr. number: %d\n"\
						"Last MAC database update time: %s", status, cevap_vermeyen_IP_sayisi, ayarlar.LastUpdateTime);
			syslog(LOG_INFO, buf);
			syslog(LOG_INFO, "MTAHost:%s, From:%s, to:%s", ayarlar.MTAHost, ayarlar.FromName, ayarlar.To1);
			simple_send_mail(ayarlar.MTAHost, ayarlar.FromName, ayarlar.To1, "trackerd OK", buf);
			report=0; /* Do not send this report again */
			cevap_vermeyen_IP_sayisi=0;
		}

		/*  You may send the report if ... */
		if( !report && (tmb->tm_hour != TIME1  && tmb->tm_hour != TIME2)  ) report=1;
	
		/* Check if tracking process crashed, if crashed set do like first start  */
		trackerpidfile=fopen(TRACKERPIDFILE, "r");
		if( trackerpidfile ) {
			fscanf(trackerpidfile, "%s", trackerpid_str);
			trackerpid=atoi(trackerpid_str);
			/* If unable to send the signal 0, it means that there is no such process,
				so process was crashed without removing its pid file
			*/
			if( kill((pid_t)trackerpid, 0) ) first_start=1;
			fclose(trackerpidfile);
		}

		if( first_start || !strcmp(ayarlar.Status, "sleeping") ) {
			/* convert the LastUpdateFinishTime (in the database) into the timetick value */
			#define tmp ayarlar.LastUpdateFinishTime
			year[0]=tmp[0]; year[1]=tmp[1]; year[2]=tmp[2]; year[3]=tmp[3]; year[4]=(char)NULL;
			month[0]=tmp[5]; month[1]=tmp[6]; month[2]=(char)NULL;
			day[0]=tmp[8]; day[1]=tmp[9]; day[2]=(char)NULL;
			hour[0]=tmp[11]; hour[1]=tmp[12]; hour[2]=(char)NULL;
			minute[0]=tmp[14]; minute[1]=tmp[15]; minute[2]=(char)NULL;
			time_struct.tm_year=atoi(year)-1900; time_struct.tm_mon=atoi(month)-1;
			time_struct.tm_mday=atoi(day);  time_struct.tm_hour=atoi(hour);
			time_struct.tm_min=atoi(minute); time_struct.tm_sec=0;
			/* Add the TrackerPeriod time */
			NewRunTime_timetick=mktime( &time_struct ) + ayarlar.TrackerPeriod*60;
			time(&curtime); 
			if( NewRunTime_timetick < curtime || first_start ) {
				first_start=0;
				if( (pid=fork())<0) 
					syslog(LOG_ERR, "main: tracking process fork() error %m");
				/* In the child process pid=0, in the main process pid=number */
				else if( pid ) { 
					(void)signal(SIGINT, cik);
					(void)signal(SIGTERM, cik);
					(void)signal(SIGHUP, cik);
					(void)signal(SIGQUIT, cik);
					openlog("trackerd", LOG_NOWAIT|LOG_PID,  LOG_LOCAL1);
					setlogmask( LOG_UPTO(LOG_INFO) );
					#ifdef debug
					setlogmask( LOG_UPTO(LOG_DEBUG) );
					#endif
					trackerpidfile=fopen(TRACKERPIDFILE, "w");
					if( !trackerpidfile ) {
						syslog(LOG_ERR, "Couldn't create %s for writing !\n",TRACKERPIDFILE);
						exit(EXIT_FAILURE);
					} 
					sprintf( buf, "%d", getpid() );
					fprintf(trackerpidfile, buf);
					fclose(trackerpidfile);
					tracker();
					remove(TRACKERPIDFILE);
					exit(EXIT_SUCCESS);
				}
			}
		}

		if( !wait_counter ) syslog(LOG_INFO, "main: Waiting %d seconds for the new device cheking process", ayarlar.Period);
		sleep( WAIT ); wait_counter+=WAIT;
	}while(daemon);
	
	exit(0);
}

int isle(struct ayr *ayarlar)
{
    int ipno=0, cevap, error=0;
    static char line[256], host[256];
	FILE *cevapvar;
	FILE *cevapyok;
	FILE *listeyaz;

	cevapvar=fopen(CEVAPVAR, "w");
	cevapyok=fopen(CEVAPYOK, "w");
	listeyaz=fopen(LISTE, "r");

	while(fgets(line,132,listeyaz)) {
    	if (sscanf(line,"%s",host) != 1) continue;
        if ((!*host) || (host[0]=='#')) continue; /* magic to avoid comments */
		ipno++;
		cevap=pingthost( host, PING_TIMEOUT );
	    if( cevap>0 ) fprintf(cevapvar, "%s is alive\n", host);
        else if( !snmp(host, ayarlar) ) { /* snmp'den de cevap gelmediyse yaz */
			fprintf(cevapyok, "%s\n", host);
			error=1;
		}
		syslog(LOG_INFO, "isle: host[%d]:%s, cevap=%d", ipno, host, cevap);
	}
	fclose(cevapvar);
	fclose(cevapyok);
	fclose(listeyaz);
	
	return error;
}
 
int sendmail(struct ayr *ayarlar)
{
  	char mesaj[1024], buf[255], ipaddress[4], *dns;
  	int i=0, num=0;
	FILE *cevapyok=fopen(CEVAPYOK, "r");
	mesaj[0]=(char)NULL;
	

	while( !feof(cevapyok) ) {
	  	buf[i]=(char)fgetc( cevapyok );
		if( buf[i++]=='\n' ) {
            		num++; // Adds DNS name of the IP address to the mesaj string
			dns=cevir(ipaddress, buf);
			syslog(LOG_INFO,"dns=%s\n", dns);
			strcat( mesaj, "\n");
			strcat( mesaj, dns );
			i=0;
            		if( strlen(mesaj) > 900 ) {
                		strcat( mesaj, " Too many devices...");
                		break;
            		}
        	}
	}
	
	fclose(cevapyok);

	if( strlen(mesaj)<5 ) {
		syslog(LOG_INFO, "sendmail:cevap vermeyen cihaz yok\n");
		return 0;
	}
	
	syslog(LOG_INFO, "sendmail: Cevap vermeyen cihazlar:\n%s\n", mesaj);
	syslog(LOG_INFO, "sendmail:from:%s, kime:%s, MTAHost:%s, Subj:%s\n", ayarlar->FromName, ayarlar->To1, ayarlar->MTAHost, ayarlar->Subject);
	simple_send_mail(ayarlar->MTAHost, ayarlar->FromName, ayarlar->To1, ayarlar->Subject, mesaj);
	return num;
}

char *cevir(char *fin, char *address)
{
	struct hostent *host;
	char buf[32];
	static char sonuc[255];
	int i = 0, j = 0, num = 0, exit=0;
	        
        do {
		if( (*(address + i) == '.') || (*(address + i) == (char)NULL) ) {
			buf[j]= (char)NULL;
			fin[num]=atoi(buf);
			//printf("buf=%s, fin[%d]=%d\n", buf, num, fin[num]);
			num++; j=-1; 
			if( *(address + i) == (char)NULL ) exit=1;
		}
		else buf[j] = *(address + i);
		++i; j++;
	}while( !exit );
	host=gethostbyaddr(fin, 4, AF_INET);
	if( !host ) {
	 // Coldn't resolv
    		strncpy(sonuc, address, 254);
  	}
  	else strncpy(sonuc, host->h_name, 254);
	return sonuc;
}


void cik(int signo)
{
	syslog(LOG_INFO, "cik: trackerd ended.");
	closelog();
	exit(1);
}

int snmp(char *host, struct ayr *ayarlar)
{
    struct snmp_session session, *ss;
    struct snmp_pdu *pdu;
    struct snmp_pdu *response;

    oid anOID[MAX_OID_LEN];
    size_t anOID_len = MAX_OID_LEN;

    int status;
    
    /*
     * Initialize the SNMP library
     */
    init_snmp("snmpapp");

    /*
     * Initialize a "session" that defines who we're going to talk to
     */
    snmp_sess_init( &session ); /* set up defaults */
    session.peername = strdup(host);

    /* set the SNMP version number */
    session.version = SNMP_VERSION_2c;

    /* set the SNMPv1 community name used for authentication */
    session.community = (u_char *)ayarlar->SnmpRO;
    session.community_len = (unsigned) strlen((char *)session.community);

    /*
     * Open the session
     */
    SOCK_STARTUP;
    ss = snmp_open(&session); /* establish the session */

    if (!ss) {
        snmp_perror("ack");
        snmp_log(LOG_ERR, "something horrible happened!!!\n");
        return 0;
    }
    
    /*
     * Create the PDU for the data for our request.
     *   1) We're going to GET the system.sysDescr.0 node.
     */
    pdu = snmp_pdu_create(SNMP_MSG_GET);
    read_objid(".1.3.6.1.2.1.1.1.0", anOID, &anOID_len);

    snmp_add_null_var(pdu, anOID, anOID_len);
  
    /*
     * Send the Request out.
     */
    status = snmp_synch_response(ss, pdu, &response);

    if(response) snmp_free_pdu(response);
    snmp_close(ss);

    SOCK_CLEANUP;
    return(status==STAT_SUCCESS ? 1 : 0);
}
