/* Network Trackerd
Copyright Ali BASEL
Ali BASEL <alib@sabanciuniv.edu>, <ali@basel.name.tr>
http://people.sabanciuniv.edu/alib
http://www.basel.name.tr
2004.03.22
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
*/

#include "tracker.h"


int mysql_query_trackerd(char *query, struct ayr *ayarlar)
{
	MYSQL mysql;
	int num_rows=0;
	
	if((mysql_thread_init() != 0)) {
		syslog(LOG_INFO, "mysql_thread_init failed");
		abort();
	}

	mysql_init(&mysql);

	if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_INFO, "Failed to connect to database: Error: %s\n", mysql_error(&mysql));
		mysql_thread_end();
		return -1;
	}
	
	if( mysql_real_query(&mysql, query, strlen(query)) ) {
		syslog(LOG_ERR, "MySQL Error:%s. Query=%s\n", mysql_error(&mysql), query);
	}
    else { /* query applied */
		num_rows=mysql_affected_rows(&mysql);
		#ifdef debug
		syslog(LOG_DEBUG, "Affected num_rows:%d Query applied:%s\n", num_rows, query);
		#endif
    }
    mysql_close(&mysql);
	mysql_thread_end();
	return num_rows;
}

int calculate_statistics_ap(struct ayr *ayarlar, char *zaman)
{
    MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
	char query[2048];
	char LastUpdateTime[20]; /* Current value of the LastUpdateTime field in the Stats table */
    char LastUpdateFinishTime[20], oldest_time[20];
	int CurrentTotalMACNumber=0;
	int TotalMACNumber=0;
	int PeakTotalMACNumber=0;
	char PeakTotalMACNumberDate[20];
	int num_rows;
    time_t curtime;
	struct tm *tmb;

	mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
        syslog(LOG_ERR, "AP calculate_statistics: Failed to connect to database: Error: %s\n", mysql_error(&mysql));
        return 0;
    }

	sprintf(query, "select LastUpdateTime, PeakTotalMACNumber, PeakTotalMACNumberDate from APStats");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "AP calculate_statistics: Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
		mysqlresult = mysql_store_result(&mysql);
		num_rows= mysql_num_rows(mysqlresult);
		if( !num_rows ) { /* If Stats table is cleared */
			strncpy(LastUpdateTime, zaman, 20);
			PeakTotalMACNumber=0;
			strncpy(PeakTotalMACNumberDate, zaman, 20);
		}
		else {
			mysqlrow = mysql_fetch_row(mysqlresult);
			strncpy(LastUpdateTime, mysqlrow[0], 20);
			PeakTotalMACNumber=atoi(mysqlrow[1]);
			strncpy(PeakTotalMACNumberDate, mysqlrow[2], 20);
		}
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}
	
	/* Find the current total wireless MAC number */
	sprintf(query, "select count(*) from macs where time='%s' and ifname='AP'", LastUpdateTime);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "AP calculate_statistics: Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
		mysqlresult = mysql_store_result(&mysql);
		mysqlrow = mysql_fetch_row(mysqlresult);
		CurrentTotalMACNumber=atoi(mysqlrow[0]);
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}
	

	if( CurrentTotalMACNumber >= PeakTotalMACNumber ) {
		PeakTotalMACNumber=CurrentTotalMACNumber;
		strncpy(PeakTotalMACNumberDate, LastUpdateTime, 17);
	}
		
	/* Find the total wireless MAC number in the database*/
	sprintf(query, "select distinct mac from macs where ifname='AP'");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_INFO, "AP calculate_statistics: Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
		mysqlresult = mysql_store_result(&mysql);
		TotalMACNumber = mysql_num_rows(mysqlresult);
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}
	

	/* Tracking isleminin bittigi zamani belirle */
    time(&curtime); tmb=localtime(&curtime);
    sprintf(LastUpdateFinishTime, "%4d.%02d.%02d-%02d:%02d", tmb->tm_year+1900, tmb->tm_mon+1, tmb->tm_mday, tmb->tm_hour, tmb->tm_min);        
    syslog(LOG_INFO, "AP calculate_statistics: LastUpdateFinishTime:%s", LastUpdateFinishTime);

		
    sprintf(query, "update APStats set Status='sleeping', LastUpdateFinishTime='%s', CurrentTotalMACNumber=%d, PeakTotalMACNumber=%d, PeakTotalMACNumberDate='%s', TotalMACNumber=%d", LastUpdateFinishTime, CurrentTotalMACNumber, PeakTotalMACNumber, PeakTotalMACNumberDate, TotalMACNumber);
    #ifdef debug
        syslog(LOG_INFO, "AP calculate_statistics: %s", query);
    #endif
    if( mysql_real_query(&mysql, query, strlen(query)) )
        syslog(LOG_INFO, "AP calculate_statistics: MySQL Error:%s, Query=%s", mysql_error(&mysql), query);
    else { // update query applied
        num_rows=mysql_affected_rows(&mysql);
        syslog(LOG_INFO, "AP calculate_statistics: APStats udated,query=%s\n", query);
    }

	/* Count the records in the LocalStats table */
	sprintf(query, "select UpdateTime from APLocalStats order by 1");
	#ifdef debug
	syslog(LOG_INFO, query);
	#endif
	if( mysql_real_query(&mysql, query, strlen(query)) ) {
		syslog(LOG_INFO, "Error:%s, Query:%s", mysql_error(&mysql), query);
	}
	else { // query succeeded, process any data returned by it
		mysqlresult = mysql_store_result(&mysql);
		num_rows = mysql_num_rows(mysqlresult);
		syslog(LOG_INFO, "AP LocalStats is found %d times in the database", num_rows);
		/* the number of LocalStats in the database is less than 'LocalHistoryCount', so, insert the record */
		if( num_rows < ayarlar->LocalHistoryCount ) {
			sprintf(query, "insert into APLocalStats values ('%s', %d)", LastUpdateTime, CurrentTotalMACNumber);
			#ifdef debug
			syslog(LOG_INFO, query);
			#endif
			if( mysql_real_query(&mysql, query, strlen(query)) ) {
				syslog(LOG_INFO, "MySQL Error:%s. APLocalStats insert query failed ! Query=%s", mysql_error(&mysql), query);
			}
			else { /* insert query applied */
				num_rows=mysql_affected_rows(&mysql);
				if( num_rows ) { // insert basarili, geriye bir data gelmez.
					syslog(LOG_INFO, "APLocalStats inserted. Query:%s\n", query);
					if( mysqlresult ) mysql_free_result(mysqlresult);
				}
				else { /* insert basarisiz */
					syslog(LOG_INFO, "APLocalStats couldn't been inserted ! Query=%s", query);
				}
			}
		} /* end of insert query */
		else {  /* There are 5 records in the database */
			/* find the oldest record and update it */
			mysqlrow = mysql_fetch_row(mysqlresult);
			strcpy(oldest_time, mysqlrow[0]);
			/* clear the current query */
			mysql_free_result(mysqlresult);
			/* update the record wich has the oldest time */
			sprintf(query, "update APLocalStats set UpdateTime='%s', TrackedMACNumber=%d where UpdateTime='%s'", LastUpdateTime, CurrentTotalMACNumber, oldest_time);
			#ifdef debug
			syslog(LOG_INFO, query);
			#endif
			if( mysql_real_query(&mysql, query, strlen(query)) ) {
				syslog(LOG_INFO, "MySQL Error:%s. APLocalStats update query failed ! Query=%s", mysql_error(&mysql), query);
			}
			else {
				/* update query applied */
				num_rows=mysql_affected_rows(&mysql);
				if(  num_rows ) /* update basarili, geriye bir data gelmez. */
					syslog(LOG_INFO, "APLocalStats updated,query=%s\n", query);
				else syslog(LOG_INFO, "APLocalStats update failed: query=%s\n", query);
			}
		}
	}

	mysql_close(&mysql);
	return 1;
}


int calculate_statistics(struct ayr *ayarlar, char *zaman)
{
    MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
	char query[2048];
	char LastUpdateTime[20]; /* Current value of the LastUpdateTime field in the Stats table */
    char LastUpdateFinishTime[20], oldest_time[20];
	int CurrentTotalMACNumber=0;
	int TotalMACNumber=0;
	int PeakTotalMACNumber=0;
	char PeakTotalMACNumberDate[20];
	int num_rows;
    time_t curtime;
	struct tm *tmb;

	mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
        syslog(LOG_INFO, "calculate_statistics: Failed to connect to database: Error: %s\n", mysql_error(&mysql));
        return 0;
    }

	sprintf(query, "select LastUpdateTime, PeakTotalMACNumber, PeakTotalMACNumberDate from Stats");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_INFO, "calculate_statistics: Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
		mysqlresult = mysql_store_result(&mysql);
		num_rows= mysql_num_rows(mysqlresult);
		if( !num_rows ) { /* If Stats table is cleared */
			strncpy(LastUpdateTime, zaman, 20);
			PeakTotalMACNumber=0;
			strncpy(PeakTotalMACNumberDate, zaman, 20);
		}
		else {
			mysqlrow = mysql_fetch_row(mysqlresult);
			strncpy(LastUpdateTime, mysqlrow[0], 20);
			PeakTotalMACNumber=atoi(mysqlrow[1]);
			strncpy(PeakTotalMACNumberDate, mysqlrow[2], 20);
		}
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}
	
	/* Find the current total MAC number */
	sprintf(query, "select count(*) from macs where time='%s' and ifname!='AP'", LastUpdateTime);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_INFO, "calculate_statistics: Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
		mysqlresult = mysql_store_result(&mysql);
		mysqlrow = mysql_fetch_row(mysqlresult);
		CurrentTotalMACNumber=atoi(mysqlrow[0]);
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}
	

	if( CurrentTotalMACNumber >= PeakTotalMACNumber ) {
		PeakTotalMACNumber=CurrentTotalMACNumber;
		strncpy(PeakTotalMACNumberDate, LastUpdateTime, 17);
	}
		
	/* Find the total MAC number in the database*/
	sprintf(query, "select distinct mac from macs");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_INFO, "calculate_statistics: Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
		mysqlresult = mysql_store_result(&mysql);
		TotalMACNumber = mysql_num_rows(mysqlresult);
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}
	

	/* Tracking isleminin bittigi zamani belirle */
    time(&curtime); tmb=localtime(&curtime);
    sprintf(LastUpdateFinishTime, "%4d.%02d.%02d-%02d:%02d", tmb->tm_year+1900, tmb->tm_mon+1, tmb->tm_mday, tmb->tm_hour, tmb->tm_min);        
    syslog(LOG_INFO, "calculate_statistics: LastUpdateFinishTime:%s", LastUpdateFinishTime);

		
    sprintf(query, "update Stats set Status='sleeping', LastUpdateFinishTime='%s', CurrentTotalMACNumber=%d, PeakTotalMACNumber=%d, PeakTotalMACNumberDate='%s', TotalMACNumber=%d", LastUpdateFinishTime, CurrentTotalMACNumber, PeakTotalMACNumber, PeakTotalMACNumberDate, TotalMACNumber);
    #ifdef debug
        syslog(LOG_INFO, "calculate_statistics:%s", query);
    #endif
    if( mysql_real_query(&mysql, query, strlen(query)) )
        syslog(LOG_INFO, "calculate_statistics: MySQL Error:%s, Query=%s", mysql_error(&mysql), query);
    else { // update query applied
        num_rows=mysql_affected_rows(&mysql);
        syslog(LOG_INFO, "calculate_statistics: Stats udated,query=%s\n", query);
    }

	/* Count the records in the LocalStats table */
	sprintf(query, "select UpdateTime from LocalStats order by 1");
	#ifdef debug
	syslog(LOG_INFO, query);
	#endif
	if( mysql_real_query(&mysql, query, strlen(query)) ) {
		syslog(LOG_INFO, "databaze_yaz: Error:%s, Query:%s", mysql_error(&mysql), query);
	}
	else { // query succeeded, process any data returned by it
		mysqlresult = mysql_store_result(&mysql);
		num_rows = mysql_num_rows(mysqlresult);
		syslog(LOG_INFO, "LocalStats is found %d times in the database", num_rows);
		/* the number of LocalStats in the database is less than 'LocalHistoryCount', so, insert the record */
		if( num_rows < ayarlar->LocalHistoryCount ) {
			sprintf(query, "insert into LocalStats values ('%s', %d)", LastUpdateTime, CurrentTotalMACNumber);
			#ifdef debug
			syslog(LOG_INFO, query);
			#endif
			if( mysql_real_query(&mysql, query, strlen(query)) ) {
				syslog(LOG_INFO, "MySQL Error:%s. LocalStats insert query failed ! Query=%s", mysql_error(&mysql), query);
			}
			else { /* insert query applied */
				num_rows=mysql_affected_rows(&mysql);
				if( num_rows ) { // insert basarili, geriye bir data gelmez.
					syslog(LOG_INFO, "LocalStats inserted. Query:%s\n", query);
					if( mysqlresult ) mysql_free_result(mysqlresult);
				}
				else { /* insert basarisiz */
					syslog(LOG_INFO, "LocalStats couldn't been inserted ! Query=%s", query);
				}
			}
		} /* end of insert query */
		else {  /* There are 5 records in the database */
			/* find the oldest record and update it */
			mysqlrow = mysql_fetch_row(mysqlresult);
			strcpy(oldest_time, mysqlrow[0]);
			/* clear the current query */
			mysql_free_result(mysqlresult);
			/* update the record wich has the oldest time */
			sprintf(query, "update LocalStats set UpdateTime='%s', TrackedMACNumber=%d where UpdateTime='%s'", LastUpdateTime, CurrentTotalMACNumber, oldest_time);
			#ifdef debug
			syslog(LOG_INFO, query);
			#endif
			if( mysql_real_query(&mysql, query, strlen(query)) ) {
				syslog(LOG_INFO, "MySQL Error:%s. LocalStats update query failed ! Query=%s", mysql_error(&mysql), query);
			}
			else {
				/* update query applied */
				num_rows=mysql_affected_rows(&mysql);
				if(  num_rows ) /* update basarili, geriye bir data gelmez. */
					syslog(LOG_INFO, "LocalStats updated,query=%s\n", query);
				else syslog(LOG_INFO, "LocalStats update failed: query=%s\n", query);
			}
		}
	}


	mysql_close(&mysql);
	return 1;
}


char **router_listesi_al(int *switch_sayisi, struct ayr *ayarlar)
{
    MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
    int i, num_rows_Main=0, num_rows_Modules=0;
    char query[500], **switchler=NULL;

    mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_ERR, "router_listesi_al: Failed to connect to database: Error: %s\n", mysql_error(&mysql));
        mysql_close(&mysql); *switch_sayisi=0; return 0;
    }

    // Find total number of routers for allocating memory= num_rows_Main + num_rows_Modules
    sprintf(query, "select IP from Main where tracking='router' and IP not like '#%%'");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Switch listesi search query'de hata olustu ! Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows_Main = mysql_num_rows(mysqlresult);
        if( mysqlresult ) mysql_free_result(mysqlresult);
     }

     
    sprintf(query, "select IP1 from Modules where tracking='router' and IP1 not like '#%%'");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Router listesi search query'de hata olustu ! Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows_Modules = mysql_num_rows(mysqlresult);
        *switch_sayisi=num_rows_Main+num_rows_Modules;
        syslog(LOG_INFO, "num_rows_Modules:%d, num_rows_Main:%d, Total Routers:%d", num_rows_Modules, num_rows_Main, *switch_sayisi);
        switchler = (char **)malloc(*switch_sayisi*sizeof(char *) );
        if( !switchler ) {
            syslog(LOG_ERR, "Couldn't allocate memory for Routers! num_rows_Modules=%d", num_rows_Modules); exit(0);
        }
        for(i=0; i<num_rows_Modules; i++) {
            switchler[i]=(char *)malloc(16); // Memory for one IP address
		    mysqlrow = mysql_fetch_row(mysqlresult);
            strncpy(switchler[i], mysqlrow[0], 15);
        }
	    if( mysqlresult ) mysql_free_result(mysqlresult);
     }

    sprintf(query, "select IP from Main where tracking='router' and IP not like '#%%'");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Router listesi search query'de hata olustu ! Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows_Main = mysql_num_rows(mysqlresult);
        for(i=num_rows_Modules; i<*switch_sayisi; i++) {
            switchler[i]=(char *)malloc(16); // Memory for one IP address
		    mysqlrow = mysql_fetch_row(mysqlresult);
            strncpy(switchler[i], mysqlrow[0], 15);
        }
	    if( mysqlresult ) mysql_free_result(mysqlresult);
     }
	mysql_close(&mysql);
    return switchler;
}



char **switch_listesi_al(char *key, int *switch_sayisi, struct ayr *ayarlar, char *zaman)
{
    MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
    int num_rows=0, i;
    char query[500], **switchler=NULL;
    
    mysql_init(&mysql);

    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_ERR, "switch_listesi_al: Failed to connect to database: Error: %s\n", mysql_error(&mysql));
        mysql_close(&mysql); *switch_sayisi=num_rows; return 0;
    }

    /* key=on for switches, key=router for routers-arp table */
    sprintf(query, "select IP from Main where tracking='%s' and IP not like '#%%'", key);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "switch_listesi_al: Error:%s, Switch listesi search query'de hata olustu ! Query=%s", mysql_error(&mysql), query);
    }
    else { /* query succeeded, process any data returned by it */
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        *switch_sayisi=num_rows;
        switchler=(char **)malloc(num_rows*sizeof(char *) );
        if( !switchler ) {
            syslog(LOG_ERR, "switch_listesi_al: Couldn't allocate memory for switchler! num_switchler=%d", num_rows); exit(0);
        }
        syslog(LOG_INFO, "switch_listesi_al: Total Number of Switches to Track:%d", num_rows);
        for(i=0; i<num_rows; i++) {
		    mysqlrow = mysql_fetch_row(mysqlresult);
            switchler[i]=(char *)malloc(16); // Memory for one IP address
            strncpy(switchler[i], mysqlrow[0], 15);
        }
	    if( mysqlresult ) mysql_free_result(mysqlresult);
     }
		
	if( !strcmp(key, "apon") ) sprintf(query, "update APStats set LastUpdateTime='%s', Status='running'", zaman);
	else sprintf(query, "update Stats set LastUpdateTime='%s', Status='running'", zaman);
	strcpy(ayarlar->LastUpdateTime, zaman);
	strcpy(ayarlar->Status, "running");

    #ifdef debug
        syslog(LOG_INFO, query);
    #endif
    if( mysql_real_query(&mysql, query, strlen(query)) ) 
        syslog(LOG_INFO, "switch_listesi_al: MySQL Error:%s, Query=%s", mysql_error(&mysql), query);
    else { // update query applied
        num_rows=mysql_affected_rows(&mysql);
        syslog(LOG_INFO, "switch_listesi_al: Query=%s\n", query);
		if( !num_rows ) { /* If stats table is empty */
			syslog(LOG_INFO, "Stats table is empty!\n");
			if( !strcmp(key, "apon") ) sprintf(query, "insert into APStats values ('running', '%s', '%s', 0, 0, '%s', 0)", zaman, zaman, zaman);
			else sprintf(query, "insert into Stats values ('running', '%s', '%s', 0, 0, '%s', 0)", zaman, zaman, zaman);
			if( mysql_query_trackerd(query, ayarlar)==-1 ) exit(0);
		}
    }

	mysql_close(&mysql);
    return switchler;
}


int IPListAlYaz(struct ayr *ayarlar)
{
    MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
    int num_rows, ipsayisi=0;
    int i, ii;
    char query[500];
	FILE *listeyaz=fopen(LISTE, "w");


    mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_INFO, "IPListAlYaz: Failed to connect to database: Error: %s\n", mysql_error(&mysql));
        return 0;
    }

    sprintf(query, "select IP from Main");
	#ifdef debug
    	syslog(LOG_INFO, query);
	#endif
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
    	syslog(LOG_INFO, "Error:%s, Query:%s", mysql_error(&mysql), query);
		return 0;
    }
    else { // query succeeded, process any data returned by it
    	mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
		#ifdef debug
		syslog(LOG_INFO, "IPListAlYaz MAIN: Bulanan Kayit Sayisi:%d", num_rows);
		#endif
    	for(i=0; i<num_rows; i++) {
			mysqlrow = mysql_fetch_row(mysqlresult);
			if( !mysqlrow[0] ) {
				#ifdef debug
				syslog(LOG_INFO, "IP[%d]= is null", i);
				#endif
				continue;
			}
			#ifdef debug
			syslog(LOG_INFO,"Main IP[%d]:%s", i, mysqlrow[0]);  
			#endif
			if( strlen(mysqlrow[0])>4 && mysqlrow[0][0]!='#' ) {
				fprintf(listeyaz, "%s\n", mysqlrow[0] );
      			ipsayisi++;
			}
		}
		mysql_free_result(mysqlresult);
    }

    sprintf(query, "select IP1, IP2, IP3, IP4, IP5, IP6, IP7, IP8, IP10, IP11, IP12, IP13, IP14, IP15 from Modules");
	#ifdef debug
	syslog(LOG_INFO, query);
	#endif
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
    	syslog(LOG_INFO, "Error:%s, Query:%s", mysql_error(&mysql), query);
		return 0;
    }
    else { // query succeeded, process any data returned by it
    	mysqlresult = mysql_store_result(&mysql);
		num_rows = mysql_num_rows(mysqlresult);
		#ifdef debug
        syslog(LOG_INFO, "IPListAlYaz: MODUL --bulanan Kayit Sayisi:%d", num_rows);
		#endif
	    for(i=0; i<num_rows; i++) {
			mysqlrow = mysql_fetch_row(mysqlresult);
			for(ii=0; ii<13; ii++) {
				if( !mysqlrow[ii] ) {
					#ifdef debug
					syslog(LOG_INFO, "field[%d][%d]= is null", i, ii); 
					#endif
					continue;
				}
				#ifdef debug
				syslog(LOG_INFO, "field[%d][%d], len=%d", i, ii, strlen(mysqlrow[ii]) );
				#endif
				if( strlen(mysqlrow[ii])>4 && mysqlrow[ii][0]!='#' ) {
					fprintf(listeyaz, "%s\n", mysqlrow[ii] );
					ipsayisi++;
				}
			}
		}
		mysql_free_result(mysqlresult);
	}
    mysql_close(&mysql);

	fclose(listeyaz);
	syslog(LOG_INFO, "Listeye eklenen Ip Sayisi:%d", ipsayisi);
	return ipsayisi;
}


int Ayarlar(struct ayr *ayarlar)
{
    MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
    unsigned num_rows, i;
    int status=0;
    char query[500];

    mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_INFO, "Settings: Failed to connect to database: Error: %s\n", mysql_error(&mysql));
        return 0;
    }

    sprintf(query, "select * from Settings");
    #ifdef debug
        syslog(LOG_INFO, query);
    #endif
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
    	syslog(LOG_INFO, "Error:%s, Query:%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
    	mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        #ifdef debug
            syslog(LOG_INFO, "Settings: Bulanan Kayit Sayisi:%d", num_rows);
        #endif
        if (!num_rows) { // there are no rows
	    	status=-1;
        }
        else { //yani boyle bir kayit varsa
	    	for(i=0; i<num_rows; i++) {
				mysqlrow = mysql_fetch_row(mysqlresult);
	      	    strcpy(ayarlar->MTAHost, mysqlrow[0]);
				strcpy(ayarlar->FromName, mysqlrow[1]);
				strcpy(ayarlar->To1, mysqlrow[2]);
				if( mysqlrow[3] ) strcpy(ayarlar->To2, mysqlrow[3]);				
				else ayarlar->To2[0]=(char)NULL;
				if( mysqlrow[4] ) strcpy(ayarlar->To3, mysqlrow[4]);				
				else ayarlar->To3[0]=(char)NULL;
				strcpy(ayarlar->Subject, mysqlrow[5]);				
				ayarlar->Period=atoi( mysqlrow[6] )*60;	// dakika yapiyor;
				ayarlar->TrackerPeriod=atoi( mysqlrow[7] );
				strcpy(ayarlar->Tracking, mysqlrow[8]);
				strcpy(ayarlar->SnmpRO, mysqlrow[9]);
				strcpy(ayarlar->SnmpRW, mysqlrow[10]);
				ayarlar->HistoryCount=atoi( mysqlrow[11] );
				ayarlar->LocalHistoryCount=atoi( mysqlrow[12] );
	    	}
	    	status=1;
       	}
	    mysql_free_result(mysqlresult);
    }

    sprintf(query, "select Status, LastUpdateTime, LastUpdateFinishTime from Stats");
    #ifdef debug
        syslog(LOG_INFO, query);
    #endif
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
    	syslog(LOG_INFO, "Error:%s, Query:%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
    	mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        #ifdef debug
            syslog(LOG_INFO, "Settings: Bulanan Kayit Sayisi:%d", num_rows);
        #endif
        if (!num_rows) { // there are no rows
			strcpy(ayarlar->Status, "sleeping");
	    	strcpy(ayarlar->LastUpdateTime, "never");
			strcpy(ayarlar->LastUpdateFinishTime, "2005.05.05-05:05");
        }
        else { //yani boyle bir kayit varsa
			mysqlrow = mysql_fetch_row(mysqlresult);
			strcpy(ayarlar->Status, mysqlrow[0]);
			strcpy(ayarlar->LastUpdateTime, mysqlrow[1]);
			strcpy(ayarlar->LastUpdateFinishTime, mysqlrow[2]);
       	}
		mysql_free_result(mysqlresult);
    }

	mysql_close(&mysql);
	return status;
}
