/*
 * snmpwalk.c - send snmp GETNEXT requests to a network entity, walking a
 * subtree.
 *
 */
/**********************************************************************
	Copyright 1988, 1989, 1991, 1992 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/
#include <net-snmp/net-snmp-config.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif
#include <sys/types.h>
#if HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif
#if TIME_WITH_SYS_TIME
# ifdef WIN32
#  include <sys/timeb.h>
# else
#  include <sys/time.h>
# endif
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#if HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#include <stdio.h>
#if HAVE_WINSOCK_H
#include <winsock.h>
#endif
#if HAVE_NETDB_H
#include <netdb.h>
#endif
#if HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif

#include <net-snmp/net-snmp-includes.h>
#include <pthread.h>

/* Network Tracker
Copyright Ali BASEL
Ali BASEL <alib@sabanciuniv.edu>, <ali@basel.name.tr>
http://www.basel.name.tr
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
*/

#include "tracker.h"

/* Switch'deki operational VLAN listesini verir */
#define VLANOID    ".1.3.6.1.4.1.9.9.46.1.3.1.1.2"

/* community_ismi@VLAN_NBR ile bu VLAN'deki MAC->BPN listesini verir */
/* Her VLAN icin bu listeyi ayri ayri almak gerekiyor */
#define MACBPNOID	".1.3.6.1.2.1.17.4.3.1.2"

/*  BPN IfIndex mapping */
#define IFINDEXOID	".1.3.6.1.2.1.17.1.4.1.2"

/* SNMP ifindex name listesi */
#define IFNAMEOID	".1.3.6.1.2.1.31.1.1.1.1"

/* Get CDP index number with ports ifindex number */
#define CDPOID		".1.3.6.1.4.1.9.9.23.1.2.1.1.3"

/* Get ARP table on the router */
#define ARPOID		".1.3.6.1.2.1.3.1.1.2"

/* MAC addresses of the interfaces */
#define IFPHYADROID	"1.3.6.1.2.1.2.2.1.6"

/* Get associated MACS in an AP */
#define APMACOID	"1.3.6.1.4.1.9.9.273.1.2.1.1.14"

#define MAX_IFINDEX_LEN 10

struct struct_snmpresult {
	char *value;
	struct struct_snmpresult *next;
	struct struct_snmpresult *start;
};
	

char *fprint_variable_string(const oid * objid, size_t objidlen, const netsnmp_variable_list * variable);
struct struct_snmpresult *snmpwalk(char *host, char *mycommunity, char *myoid);
struct struct_snmpresult *snmp_get_and_print_tracker(char *host, void *sessp, oid *theoid, size_t theoid_len, struct struct_snmpresult *snmpresult);
int vlan_table_create(char *host);
int mac_bpn_numbers(char *data, char *host, char *vlan);
int process_result(int task, char *data, char *host, char *vlan);
void switch_thread(char *host);
void convert_host(char *host);
int all_mac_bpn_numbers(char *host);
struct struct_snmpresult *add_snmpresult(struct struct_snmpresult *snmpresult);
void remove_snmpresult(struct struct_snmpresult *snmpresult);
int find_ifnames(char *host);
int find_ifindex_numbers(char *host);
int find_cdp(char *host);
void macc(char *str);
int check_duplicate_mac(char *mac, char *zaman);
int mac_table(char **switchler, int switch_sayisi, char *zaman);
int arptable(void);
void mac_format(char *str);
int remove_ifmacs(char *host);

void ap_thread(char *host);
int ap_table_create(char *host);
int ap_macs(char *host);
int ap_mac_table(char **switchler, int switch_sayisi, char *zaman);

void gnuplot();
void common_gnuplot(FILE *dat);

static struct ayr *ayarlar;


struct struct_snmpresult *add_snmpresult(struct struct_snmpresult *snmpresult)
{
	struct struct_snmpresult *next;
	
	if(!snmpresult) {
		snmpresult=(struct struct_snmpresult *)malloc( sizeof(struct struct_snmpresult) );
    	if( !snmpresult ) {
	        syslog(LOG_ERR, "Couldn't allocate memory to initialize new snmpresult struct\n");
			exit(1);
		}
		snmpresult->start=snmpresult;
		snmpresult->next=NULL;
		return snmpresult;
	}

	next=(struct struct_snmpresult *)malloc( sizeof(struct struct_snmpresult) );
	if( !next ) {
		syslog(LOG_ERR, "Couldn't allocate memory for new snmpresult struct\n");
		exit(1);
	}
	else {
		next->start=snmpresult->start;
		next->next=NULL;
		snmpresult->next=next;
	}
	return next;
}

/* Free the chain starting from the given node*/
void remove_snmpresult(struct struct_snmpresult *snmpresult)
{
	struct struct_snmpresult *next;
	
	if(!snmpresult) return;
	while( snmpresult->next ) {
		next=snmpresult->next;
		if(snmpresult->value) free(snmpresult->value); free(snmpresult);
		snmpresult=next;
	}
	if(snmpresult->value) {
		free(snmpresult->value);
		free(snmpresult);
	}
}

int tracker(void)
{
    char zaman[100];
	char *host;
    int i, switch_sayisi=0;
    time_t curtime;
	struct tm *tmb;
    char **switchler;
	pthread_t *mythread;
  
	ayarlar=(struct ayr *)malloc( sizeof(struct ayr) );
	config_oku(ayarlar);
	Ayarlar(ayarlar);


	//gnuplot();
	//abort();

	/* If tracking field is not "on", don't do any MAC tracking, and return back */
	if( strcmp(ayarlar->Tracking, "on") ) {
        syslog(LOG_WARNING, "Setting in Tracking:%s, don't do any MAC tracking !", ayarlar->Tracking);
        return 0;
    }

	/* Set the tracking start time */
    time(&curtime); tmb=localtime(&curtime);
    sprintf(zaman, "%4d.%02d.%02d-%02d:%02d", tmb->tm_year+1900, tmb->tm_mon+1, tmb->tm_mday, tmb->tm_hour, tmb->tm_min);
    syslog(LOG_INFO, "LastUpdateStartTime:%s", zaman);

    /* Database'den tracking on olan switchlerin listesi alinir ve
	Database'de Stats tablosuna Status=running, LastUpdateTime=zaman degerleri yazilir 
	ayarlar LastUpdateTime ve status alanlarina degerler atanir */
	switchler=switch_listesi_al("on", &switch_sayisi, ayarlar, zaman);
    syslog(LOG_NOTICE, "The number of switches to track MACs:%d", switch_sayisi);	


	init_snmp("trackerd_snmp"); /* Initialize the SNMP library */
	netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_OID_OUTPUT_FORMAT,NETSNMP_OID_OUTPUT_NUMERIC);
    netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_QUICK_PRINT, 1);
    netsnmp_register_loghandler(NETSNMP_LOGHANDLER_SYSLOG, LOG_INFO);





	mythread=(pthread_t *)malloc( sizeof(pthread_t) * switch_sayisi);
	if( !mythread ) {
		exit(0);
	}

    for(i=0; i<switch_sayisi; i++) syslog(LOG_NOTICE, "Switch[%d]:%s", i, switchler[i]);

    for(i=0; i<switch_sayisi; i++) {
        host=switchler[i];
		syslog(LOG_NOTICE, "thread[%d] Switch:%s: is starting.\n", i, host);
		if( pthread_create( &mythread[i], NULL, (void *)switch_thread, (void *)host) ) {
			syslog(LOG_ERR, "SwThread[%d] Switch:%s: Error creating thread.", i, host);
		}
	}	

	for(i=0; i<switch_sayisi; i++) {
		host=switchler[i];
		syslog(LOG_NOTICE,"thread[%d] Switch:%s: is waited to join.\n", i, host);
		if ( pthread_join ( mythread[i], NULL ) ) {
			syslog(LOG_ERR, "SwThread[%d] Switch:%s: Error joining thread.", i, host);
		}
	}
	free(mythread);

	/* Populate the final macs table with entries from the sw_IP tables*/
	mac_table(switchler, switch_sayisi, zaman);

	for(i=0; i<switch_sayisi; i++) free(switchler[i]);
    free(switchler);
	
	calculate_statistics(ayarlar, zaman);


	/* Database'den tracking 'apon' olan access point'lerin listesi alinir ve
	Database'de Stats tablosuna Status=running, LastUpdateTime=zaman degerleri yazilir */
	switchler=switch_listesi_al("apon", &switch_sayisi, ayarlar, zaman);    
    syslog(LOG_NOTICE, "The number of APs to track MACs:%d", switch_sayisi);

	mythread=(pthread_t *)malloc( sizeof(pthread_t) * switch_sayisi);
	if( !mythread ) {
		exit(0);
	}

	/* In order to get updates from the process just above */
	Ayarlar(ayarlar);

    for(i=0; i<switch_sayisi; i++) syslog(LOG_NOTICE, "AP[%d]:%s", i, switchler[i]);

    for(i=0; i<switch_sayisi; i++) {
        host=switchler[i];
		syslog(LOG_NOTICE, "APthread[%d]: AP:%s is starting.\n", i, host);
		if( pthread_create( &mythread[i], NULL, (void *)ap_thread, (void *)host) ) {
			syslog(LOG_ERR, "APthread[%d] AP:%s: Error creating thread.", i, host); 
		}
	}	

	for(i=0; i<switch_sayisi; i++) {
		host=switchler[i];
		syslog(LOG_NOTICE,"APthread[%d] AP:%s is waited to join.\n", i, host);
		if ( pthread_join ( mythread[i], NULL ) ) {
			syslog(LOG_ERR, "APthread[%d] AP:%s Error joining thread.", i, host);
		}
	}

	free(mythread);

	/* Populate the final macs table with entries from the ap_IP tables*/
	ap_mac_table(switchler, switch_sayisi, zaman);

	for(i=0; i<switch_sayisi; i++) free(switchler[i]);
    free(switchler);

	/* Router'lardaki ARP tablolarini kullanarak database'e  IP'leri ekle */
	syslog(LOG_NOTICE, "calculate_statistics\n");
	calculate_statistics_ap(ayarlar, zaman);
	
	arptable();	

	syslog(LOG_NOTICE, "A Tracking Process Finished Successfully.\n\n");

	gnuplot();
	syslog(LOG_NOTICE, "gnuplot pngs prepared\n");

	free(ayarlar);
	return 1;
}


void gnuplot(void)
{
	time_t time_before_24_hours_tt, LastUpdateTime_tt; /* timetick variables */
	time_t time_before_7_days_tt;
    struct tm *time_before_24_hours_frm, time_struct;
	struct tm *time_before_7_days_frm;
	char time_before_24_hours[30];
	char time_before_7_days[30];
	char year[5], month[3], day[3], hour[3], minute[3];
	char daylanmax[10], daylancurrent[10];
	char daywlanmax[10], daywlancurrent[10];
	MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
    int num_rows, i;
    char query[MAXQUERYLEN];
	FILE *dat;
	
	/* convert the LastUpdateTime (in the database) into the timetick value */
	#define LUT ayarlar->LastUpdateTime
	year[0]=LUT[0]; year[1]=LUT[1]; year[2]=LUT[2]; year[3]=LUT[3]; year[4]=(char)NULL;
	month[0]=LUT[5]; month[1]=LUT[6]; month[2]=(char)NULL;
	day[0]=LUT[8]; day[1]=LUT[9]; day[2]=(char)NULL;
	hour[0]=LUT[11]; hour[1]=LUT[12]; hour[2]=(char)NULL;
	minute[0]=LUT[14]; minute[1]=LUT[15]; minute[2]=(char)NULL;
	time_struct.tm_year=atoi(year)-1900; time_struct.tm_mon=atoi(month)-1;
	time_struct.tm_mday=atoi(day);  time_struct.tm_hour=atoi(hour);
	time_struct.tm_min=atoi(minute); time_struct.tm_sec=0;
	LastUpdateTime_tt=mktime( &time_struct );
	#define before_24_hours_tt 24*60*60
	time_before_24_hours_tt = LastUpdateTime_tt - before_24_hours_tt;
	time_before_24_hours_frm=localtime(&time_before_24_hours_tt); /* from timetick --> formatted date value */
	#define TMB time_before_24_hours_frm
	sprintf(time_before_24_hours, "%4d.%02d.%02d-%02d:%02d", TMB->tm_year+1900, TMB->tm_mon+1, TMB->tm_mday, TMB->tm_hour, TMB->tm_min);
	time_before_7_days_tt = LastUpdateTime_tt - 7*before_24_hours_tt;
	time_before_7_days_frm=localtime(&time_before_7_days_tt); /* from timetick --> formatted date value */
	#define TM7DAYSFRM time_before_7_days_frm
	sprintf(time_before_7_days, "%4d.%02d.%02d-%02d:%02d", TM7DAYSFRM->tm_year+1900, TM7DAYSFRM->tm_mon+1, TM7DAYSFRM->tm_mday, TM7DAYSFRM->tm_hour, TM7DAYSFRM->tm_min);
	 
    mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
		syslog(LOG_ERR, "Failed to connect to database. Error: %s\n", mysql_error(&mysql));
        return;
    }

	/****************************** DAY LAN ****************************/
	/* Generate daily lan gnuplot graph 			                   */
	/* and copy the png file into www home folder 	                   */
	sprintf(query, "select TrackedMACNumber from LocalStats where UpdateTime <= '%s' and  UpdateTime>= '%s' order by TrackedMACNumber desc", LUT, time_before_24_hours);
    syslog(LOG_INFO, "Query:%s\n", query);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
		if( num_rows ) {
			mysqlrow = mysql_fetch_row(mysqlresult);
			strcpy(daylanmax, mysqlrow[0]);
		}
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}

	sprintf(query, "select UpdateTime, TrackedMACNumber from LocalStats where UpdateTime <= '%s' and UpdateTime >= '%s' order by UpdateTime desc", LUT, time_before_24_hours);
    syslog(LOG_INFO, "Query:%s\n", query);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
		if( num_rows ) {
			mysqlrow = mysql_fetch_row(mysqlresult);
			strcpy(daylancurrent, mysqlrow[1]);
		}
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}


	sprintf(query, "select * from LocalStats where UpdateTime <= '%s' and UpdateTime >= '%s'", LUT, time_before_24_hours);
    syslog(LOG_INFO, "Query:%s\n", query);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        dat=fopen("/tmp/daylandat", "w");
        for(i=0; i<num_rows; i++) {
            mysqlrow = mysql_fetch_row(mysqlresult);
                fprintf(dat, "%s %s\n", mysqlrow[0], mysqlrow[1]);
        }
        fclose(dat);
        if( mysqlresult ) mysql_free_result(mysqlresult);
    }
    dat=fopen("/tmp/daylanplot", "w");
	common_gnuplot(dat);
    fprintf(dat, "set title 'Last 24 Hours LAN USAGE' font 'helvetica,14' \n");
    fprintf(dat, "set xlabel 'Max User Count:%s                                  Current User Count:%s'  font 'helvetica,14' tc lt 4 \n", daylanmax, daylancurrent);
    fprintf(dat, "set format x '%%H'\n");
    fprintf(dat, "set xrange ['%s':'%s']\n", time_before_24_hours, LUT);
    fprintf(dat, "set output '%sdaylan.png'\n", HTTPPATH);
    fprintf(dat, "plot '/tmp/daylandat' using 1:2 with linespoints title 'User Counts', '/tmp/daylandat' using 1:2 with  impulses title 'Tracking Times'\n");
	fclose(dat);
    system("/usr/bin/gnuplot < /tmp/daylanplot");
	/****************************** DAY LAN ****************************/


	/****************************** DAY WLAN ****************************/
	/* Generate daily wlan gnuplot graph                                */
	/* and copy the png file into www home folder                       */
    sprintf(query, "select TrackedMACNumber from APLocalStats where UpdateTime <= '%s' and UpdateTime >= '%s' order by TrackedMACNumber desc", LUT, time_before_24_hours);
    syslog(LOG_INFO, "Query:%s\n", query);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
		if( num_rows ) {
			mysqlrow = mysql_fetch_row(mysqlresult);
			strcpy(daywlanmax, mysqlrow[0]);
		}
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}

	sprintf(query, "select UpdateTime, TrackedMACNumber from APLocalStats where UpdateTime <= '%s' and UpdateTime >= '%s' order by UpdateTime desc", LUT, time_before_24_hours);
    syslog(LOG_INFO, "Query:%s\n", query);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
		if( num_rows ) {
			mysqlrow = mysql_fetch_row(mysqlresult);
			strcpy(daywlancurrent, mysqlrow[1]);
		}
		if( mysqlresult ) mysql_free_result(mysqlresult);
	}

	sprintf(query, "select * from APLocalStats where UPDATETIME <= '%s' and UpdateTime >= '%s'", LUT, time_before_24_hours);
    syslog(LOG_INFO, "Query:%s\n", query);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
		dat=fopen("/tmp/daywlandat", "w");
        for(i=0; i<num_rows; i++) {
            mysqlrow = mysql_fetch_row(mysqlresult);
			fprintf(dat, "%s %s\n", mysqlrow[0], mysqlrow[1]);
		}
		fclose(dat);
        if( mysqlresult ) mysql_free_result(mysqlresult);
     }

	dat=fopen("/tmp/daywlanplot", "w");
	common_gnuplot(dat);
	fprintf(dat, "set title 'Last 24 Hours WLAN USAGE'\n");
    fprintf(dat, "set xlabel 'Max User Count:%s                                  Current User Count:%s'  font 'helvetica,14' tc lt 4 \n", daywlanmax, daywlancurrent);
    fprintf(dat, "set format x '%%H'\n");
	fprintf(dat, "set xrange ['%s':'%s']\n", time_before_24_hours, LUT);
	fprintf(dat, "set output '%sdaywlan.png'\n", HTTPPATH);
	fprintf(dat, "plot '/tmp/daywlandat' using 1:2 with linespoints title 'User Counts', '/tmp/daywlandat' using 1:2 with impulses title 'Tracking Times'\n");
	fclose(dat);
	system("/usr/bin/gnuplot < /tmp/daywlanplot");
	/****************************** DAY WLAN ****************************/


	/****************************** WEEK LAN ****************************/
	/* Generate weekly lan gnuplot graph                                */
    /* and copy the png file into www home folder                       */
    sprintf(query, "select * from LocalStats where UpdateTime <= '%s' and UpdateTime >= '%s'", LUT, time_before_7_days);
    syslog(LOG_INFO, "Query:%s\n", query);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        dat=fopen("/tmp/weeklandat", "w");
        for(i=0; i<num_rows; i++) {
            mysqlrow = mysql_fetch_row(mysqlresult);
                fprintf(dat, "%s %s\n", mysqlrow[0], mysqlrow[1]);
        }
        fclose(dat);
        if( mysqlresult ) mysql_free_result(mysqlresult);
    }
    dat=fopen("/tmp/weeklanplot", "w");
    common_gnuplot(dat);
    fprintf(dat, "set title 'LAST WEEKS LAN USAGE'\n");
    fprintf(dat, "set format x '%%d/%%A'\n");
    fprintf(dat, "set xrange ['%s':'%s']\n", time_before_7_days, LUT);
    fprintf(dat, "set output '%sweeklan.png'\n", HTTPPATH);
    fprintf(dat, "plot '/tmp/weeklandat' using 1:2 with linespoints title 'User Counts', '/tmp/weeklandat' using 1:2 with impulses title 'Tracking Times'\n");
    fclose(dat);
    system("/usr/bin/gnuplot < /tmp/weeklanplot");
	/****************************** WEEK LAN ****************************/



	/****************************** WEEK WLAN ***************************/
	/* Generate weekly wlan gnuplot graph                               */
    /* and copy the png file into www home folder                       */
    sprintf(query, "select * from APLocalStats where UpdateTime <= '%s' and UpdateTime >= '%s'", LUT, time_before_7_days);
    syslog(LOG_INFO, "Query:%s\n", query);
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        dat=fopen("/tmp/weekwlandat", "w");
        for(i=0; i<num_rows; i++) {
            mysqlrow = mysql_fetch_row(mysqlresult);
            fprintf(dat, "%s %s\n", mysqlrow[0], mysqlrow[1]);
        }
        fclose(dat);
        if( mysqlresult ) mysql_free_result(mysqlresult);
    }
    dat=fopen("/tmp/weekwlanplot", "w");
    common_gnuplot(dat);
    fprintf(dat, "set title 'LAST WEEKS WLAN USAGE'\n");
    fprintf(dat, "set format x '%%d/%%A'\n");
    fprintf(dat, "set xrange ['%s':'%s']\n", time_before_7_days, LUT);
    fprintf(dat, "set output '%sweekwlan.png'\n", HTTPPATH);
    fprintf(dat, "plot '/tmp/weekwlandat' using 1:2 with linespoints title 'User Counts', '/tmp/weekwlandat' using 1:2 with impulses title 'Tracking Times'\n");
    fclose(dat);
    system("/usr/bin/gnuplot < /tmp/weekwlanplot");
	/****************************** WEEK WLAN ****************************/

    mysql_close(&mysql);
}

void common_gnuplot(FILE *dat)
{
	fprintf(dat, "set terminal png\n");
	fprintf(dat, "set key left\n");
    fprintf(dat, "set timefmt '%%Y.%%m.%%d-%%H:%%M'\n");
    fprintf(dat, "set xdata time\n");
    fprintf(dat, "set grid\n");
	fprintf(dat, "set y2tics\n");
    fprintf(dat, "set ylabel 'Number of Users'\n");
    fprintf(dat, "set size 1.25, 0.75\n");
    fprintf(dat, "set x2label 'Generated on %s'\n", LUT);
}

void ap_thread(char *host)
{
	syslog(LOG_NOTICE,"AP thread started for AP:%s\n", host);
	/* Create ap_host table */
	/* Find operational VLAN list in the switch and fill in the table */
	if( !ap_table_create(host) ) {
		syslog(LOG_ERR, "AP:%s, ap table couldn't be created, return", host);
		return;
	}

	if( !ap_macs(host) ) {
		syslog(LOG_NOTICE, "AP:%s, mac numbers couldn't be found, return", host);
		return;
	}

	/* Remove interfaces own MACs from the sw_IP table */
	syslog(LOG_NOTICE, "AP:%s, thread terminated successfully.\n", host);
}


/* Create ap_IP table */
int ap_table_create(char *host)
{
	char query[MAXQUERYLEN], buf[100];
	
	strncpy(buf, host, sizeof(buf)-1);
	convert_host(buf);
	
	snprintf(query, MAXQUERYLEN, "drop table if exists ap_%s", buf);
	if( mysql_query_trackerd(query, ayarlar)==-1 ) return 0;

	snprintf(query, MAXQUERYLEN, "create table ap_%s ( mac char(18) not null, primary key (mac) ) TYPE=MyISAM", buf);
	if( mysql_query_trackerd(query, ayarlar)==-1 ) return 0;

	return 1;
}


/* Find MACs in the AP and insert them into the ap_host table */
int ap_macs(char *host)
{
	struct struct_snmpresult *snmpresult, *start;
    int dotcount;
    char query[MAXQUERYLEN];
    char ap_table_name[100];
	char *position_mac;
	char mac[24];

    strncpy(ap_table_name, host, sizeof(ap_table_name)-1);
    convert_host(ap_table_name);

	if((mysql_thread_init() != 0)) {
		syslog(LOG_ERR, "mysql_thread_init failed");
		abort();
	}

	/* snmpresult->value holds the snmp response values */
	start=snmpresult=snmpwalk(host, ayarlar->SnmpRO, APMACOID); /* Start of the chain */
	while( snmpresult ) {
		/* .1.3.6.1.4.1.9.9.273.1.2.1.1.14.1.6.83.85.45.78.101.116.0.14.155.167.53.236 4 */
		/* MAC is the in the last part of the OID */
		position_mac=index(snmpresult->value, ' ');
		*position_mac=(char)NULL; /* this is the end of the MAC */
		dotcount=6;
		while(dotcount>0) {
			if( *position_mac=='.' ) dotcount--;
			position_mac--;
		}
		strcpy(mac, position_mac+=2);
		macc(mac);
		#ifdef debug
		syslog(LOG_DEBUG, "AP:%s, mac:%s-\n", host, mac);
		#endif
		sprintf(query, "insert into ap_%s values ('%s')", ap_table_name, mac);
		mysql_query_trackerd(query, ayarlar);
		snmpresult=snmpresult->next;
	}
	remove_snmpresult(start); /* free the chain from the start */

    return 1;
}


/* All MACs  in the ap_IP tables inserted  into the macs table */
int ap_mac_table(char **switchler, int switch_sayisi, char *zaman)
{
	MYSQL mysql;
    MYSQL_RES *mysqlresultA=NULL;
    MYSQL_RES *mysqlresultB=NULL;
	MYSQL_ROW mysqlrowA, mysqlrowB;
	int num_rows=0, num_rowsA, num_rowsB, i, ii;
	char switchIP[100], query[MAXQUERYLEN], oldest_time[100];
	char *host;

#ifdef TRACKERDTEST
	sprintf(query, "drop table if exists macs");
	mysql_query_trackerd(query, ayarlar);

	sprintf(query, "CREATE TABLE macs ( mac char(17), ifname char(10), swip char(15), ip char(15) default 'YOK', time char(17), uid char(15) default 'YOK', ifindex int(11) default -1, vlan int(11) default -1 ) TYPE=MyISAM");
	mysql_query_trackerd(query, ayarlar);

#endif

	mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT,NULL,0)) {
		syslog(LOG_ERR, "Failed to connect to database. Error: %s\n", mysql_error(&mysql));
		return 0;
	}

	for(i=0; i<switch_sayisi; i++) {
		host=switchler[i];
		strncpy(switchIP, host, sizeof(switchIP)-1);
		convert_host(switchIP);

		sprintf(query, "select mac from ap_%s", switchIP);
		syslog(LOG_DEBUG, "AP:%s, Query:%s\n", host, query);
		if( mysql_real_query(&mysql, query, strlen(query)) ) {
	    	syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
		}
		else { // query succeeded, process any data returned by it
			mysqlresultA = mysql_store_result(&mysql);
			num_rowsA = mysql_num_rows(mysqlresultA);
			for(ii=0; ii<num_rowsA; ii++) {
				mysqlrowA = mysql_fetch_row(mysqlresultA);			
				/* if(	check_duplicate_mac(mysqlrowA[0], zaman) ) {
					syslog(LOG_INFO, "Host:%s, duplicated MAC:%s, Switch:%s, ifname:%s\n", mysqlrowA[0], host, mysqlrowA[1], host);
					continue;
				} */
				/* Count the MAC in the database */
                sprintf(query, "select mac, time from macs where mac='%s' order by 2", mysqlrowA[0]);
				#ifdef debug
                syslog(LOG_DEBUG, "AP:%s, query:%s\n", host, query);
				#endif
                if( mysql_real_query(&mysql, query, strlen(query)) ) {
					syslog(LOG_ERR, "Error:%s, Query:%s", mysql_error(&mysql), query);
                    continue;
                }
                else { /* query succeeded, process any data returned by it */
                	mysqlresultB = mysql_store_result(&mysql);
                    num_rowsB = mysql_num_rows(mysqlresultB);
                    syslog(LOG_INFO, "mac=%s is found %d times in the database", mysqlrowA[0], num_rows);
                    /* the MAC is not in the database or found less than 'HistoryCount' times, so, insert the record */
                    if( num_rowsB < ayarlar->HistoryCount ) {
                    	sprintf(query, "insert into macs values ( '%s','AP','%s','AP','%s','YOK','AP','AP')",mysqlrowA[0], host, zaman);
						num_rows=mysql_query_trackerd(query, ayarlar);
						if( num_rows==-1 ) continue;
                        else { /* insert query applied */
                        	if( num_rows>0 ) { /* insert basarili, geriye bir data gelmez. */
                        		syslog(LOG_INFO, "AP MAC inserted.\n");
								#ifdef HAVE_LDAP
								/* Finds the uid of the MAC in the LDAP database and updates the tracker database with that value */
								ldap_uid_ekle(mysqlrowA[0], zaman, ayarlar);
								#endif
                        	}
                        	else { /* insert basarisiz */
                        		syslog(LOG_INFO, "AP MAC couldn't be inserted!\n");
                        	}
                        }
					} /* end of insert query */
                    else { /* There are 5 records in the database */
                    	   /* find the oldest record and update it */
                    	mysqlrowB = mysql_fetch_row(mysqlresultB);
                        strcpy(oldest_time, mysqlrowB[1]);
						/* update the record wich has the oldest time */
                        sprintf(query, "update macs set ifname='AP', swip='%s', time='%s' where mac='%s' and time='%s'", host, zaman, mysqlrowA[0], oldest_time);
						num_rows=mysql_query_trackerd(query, ayarlar);
						if( num_rows==-1 ) continue;
                        /* update query applied */
                        if( num_rows>0 ) {/* update basarili, geriye bir data gelmez. */
                        	syslog(LOG_INFO, "AP MAC updated.\n");
							#ifdef HAVE_LDAP
							/* Finds the uid of the MAC in the LDAP database and updates the tracker database with that value */
							ldap_uid_ekle(mysqlrowA[0], zaman, ayarlar);
							#endif
						}
                        else syslog(LOG_INFO, "AP MAC couldn't be updated.\n");
                    }
					if( mysqlresultB ) mysql_free_result(mysqlresultB);
        		}
			}
			if( mysqlresultA ) mysql_free_result(mysqlresultA);
		}
	}

	mysql_close(&mysql);
	return 1;
}


void switch_thread(char *host)
{
	syslog(LOG_NOTICE,"Switch thread started for switch:%s\n", host);
	/* Create sw_vlan_host and sw_host tables */
	/* Find operational VLAN list in the switch and fill in the table */
	if( !vlan_table_create(host) ) {
		syslog(LOG_ERR, "Host:%s, vlan and switch tables couldn't be created, return", host);
		return;
	}

	if( !all_mac_bpn_numbers(host) ) {
		syslog(LOG_NOTICE, "Host:%s, macbpn numbers couldn't be found, return", host);
		return;
	}

	/* Remove interfaces own MACs from the sw_IP table */
	remove_ifmacs(host);

	/* Find IfIndexNumbers of the BPNs in the switch_host table*/
	if( !find_ifindex_numbers(host) ) {
		syslog(LOG_NOTICE, "Host:%s, ifindexnumbers couldn't be found, return", host);
		return;
	}

	/* Find IfName of the MACs */
	if( !find_ifnames(host) ) {
		syslog(LOG_NOTICE, "Host:%s, ifnames couldn't be found, return", host);
		return;
	}

	/* Find out if CDP enabled on the interface */
	find_cdp(host);

	syslog(LOG_NOTICE, "Host:%s, thread terminated successfully.\n", host);
}




int vlan_table_create(char *host)
{
	struct struct_snmpresult *snmpresult, *start;
	char query[MAXQUERYLEN], buf[100];
	char *position_lastDot;
	char *position_space;

	strncpy(buf, host, sizeof(buf)-1);
	convert_host(buf);

	snprintf(query, MAXQUERYLEN, "drop table if exists sw_vlans_%s", buf);
	if( mysql_query_trackerd(query, ayarlar)==-1 ) return 0;
	
	snprintf(query, MAXQUERYLEN, "drop table if exists sw_%s", buf);
	if( mysql_query_trackerd(query, ayarlar)==-1 ) return 0;

	snprintf(query, MAXQUERYLEN, "create table sw_vlans_%s ( vlan char(4) not null, primary key (vlan) ) TYPE=MyISAM", buf);
	if( mysql_query_trackerd(query, ayarlar)==-1 ) return 0;

	snprintf(query, MAXQUERYLEN, "create table sw_%s ( vlan char(4) not null, mac char(18) not null, bpn char(3) not null, ifindex char(10), ifname char(10), cdp char(3) default 'cdp') TYPE=MyISAM", buf);
	if( mysql_query_trackerd(query, ayarlar)==-1 ) return 0;

	/* Start of the chain */
	start=snmpresult=snmpwalk(host, ayarlar->SnmpRO, VLANOID);
	/* snmpresult->value holds the snmp response values
	   Result of the snmp query for VLAN list holds the VLAN number at the last part 
		.1.3.6.1.4.1.9.9.46.1.3.1.1.2.1.140 1 --> VLAN 140 */
	while( snmpresult ) {
		position_lastDot=rindex(snmpresult->value, (char) '.');
		position_space=rindex(snmpresult->value, (char) ' ');
		*position_space=(char) NULL;
		++position_lastDot;
		if( atoi(position_lastDot) < 1000 ) {/* skip VLANs > 1000 */
			strncpy(buf, host, sizeof(buf)-1);
			convert_host(buf);
			snprintf(query, MAXQUERYLEN, "insert into sw_vlans_%s (vlan) values ('%s')", buf, position_lastDot);
			mysql_query_trackerd(query, ayarlar);
			*position_space=' ';
		}
		snmpresult=snmpresult->next;
	}

	remove_snmpresult(start); /* free the chain from the start */
	return 1;
}


/* Find IfIndex numbers of the BPN numbers in the sw_host table */
int all_mac_bpn_numbers(char *host)
{
    MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
	struct struct_snmpresult *snmpresult, *start;
    int num_rows, i;
    char query[MAXQUERYLEN];
    char buf[100];
	char *mac_end;
	char mac[24];
	char bpn[4];

    strncpy(buf, host, sizeof(buf)-1);
    convert_host(buf);

	if((mysql_thread_init() != 0)) {
		syslog(LOG_ERR, "mysql_thread_init failed");
		abort();
	}

    mysql_init(&mysql);

    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
		syslog(LOG_ERR, "Failed to connect to database. Error: %s\n", mysql_error(&mysql));
        return 0;
    }

    sprintf(query, "select vlan from sw_vlans_%s", buf);
	#ifdef debug
    syslog(LOG_DEBUG, "Query:%s\n", query);
	#endif

    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        for(i=0; i<num_rows; i++) {
            mysqlrow = mysql_fetch_row(mysqlresult);
            /* Append VLAN number into commmunity name: abs17@10 */
            strcpy(buf, ayarlar->SnmpRO); strcat(buf, "@");  strcat(buf, mysqlrow[0]); 
			#ifdef debug
            syslog(LOG_DEBUG,"Host:%s, VLAN:%s, MACBPN listesi icin community:%s\n", host, mysqlrow[0], buf);
			#endif
			/* snmpresult->value holds the snmp response values 
				.1.3.6.1.2.1.17.4.3.1.2.0.1.230.83.164.44 82 */
			start=snmpresult=snmpwalk(host, buf, MACBPNOID); /* Start of the chain */
			while( snmpresult ) {
				mac_end=index(snmpresult->value, (char)' ');
				*mac_end=(char)NULL;
				strncpy(mac, snmpresult->value+strlen(MACBPNOID)+1, 24);
				strncpy(bpn, ++mac_end, 4);
				macc(mac); /* converts mac into hexadecimal format */
				#ifdef debug
				syslog(LOG_DEBUG, "SW-%s, VLAN:%s, MAC=%s BPN:%s\n", host, mysqlrow[0], mac, bpn );	
				#endif
				strncpy(buf, host, sizeof(buf)-1);
				convert_host(buf);
				sprintf(query, "insert into sw_%s (vlan, mac, bpn, ifindex, ifname, cdp) values ('%s','%s', '%s', 'ifindex', 'ifname', 'dis')", buf, mysqlrow[0], mac, bpn);
				mysql_query_trackerd(query, ayarlar);

				snmpresult=snmpresult->next;
			}
			remove_snmpresult(start); /* free the chain from the start */

        }
        if( mysqlresult ) mysql_free_result(mysqlresult);
     }
    mysql_close(&mysql);
	mysql_thread_end();
	return 1;
}

/* Converts '.' characters into '_' in order to have legal MySQL table names */
/* 10.10.0.3 --> 10_10_0_3 */
void convert_host(char *host)
{
	char *ptr=host;

	while( *ptr ) {
		if( *ptr== '.' ) *ptr='_';
		ptr++;
	}
}

/* Remove interfaces own MAC address from the sw_IP table */
int remove_ifmacs(char *host)
{
	struct struct_snmpresult *snmpresult, *start;
	char query[MAXQUERYLEN], sw_table_name[100];
	char *mac_start;
	char mac[19];

	strncpy(sw_table_name, host, sizeof(sw_table_name)-1);
	convert_host(sw_table_name);

	/* Start of the chain */
	start=snmpresult=snmpwalk(host, ayarlar->SnmpRO, IFPHYADROID);
	/* snmpresult->value holds the snmp response values 
		.1.3.6.1.2.1.2.2.1.6.4 0:30:80:46:7a:0 */
	while( snmpresult ) {
		mac_start=index(snmpresult->value, (char)' ');
		if( ++mac_start ) {/* there is no mac in the result */
			strncpy(mac, mac_start, 19);
			mac_format( mac );
			sprintf(query, "delete from sw_%s where mac='%s'", sw_table_name, mac);
			mysql_query_trackerd(query, ayarlar);
		}
		snmpresult=snmpresult->next;
	}

	remove_snmpresult(start); /* free the chain from the start */
	return 1;
}

/* Fill the format "0:b:66:f:68:15" --> 00:0b:66:0f:68:15 */
void mac_format(char *str)
{
	int i, cnt=0, len;
	char *ptr=str;

	while( *ptr ) {
		if( *ptr==':' ) {
			if( cnt==1 ) {
				len=strlen( ptr );
				for(i=len; i>-1; i--) ptr[i]=ptr[i-1];
				ptr[i]='0';
				ptr+=2; cnt=0;
				continue;
			}
			cnt=-1;
		}
		ptr++; cnt++;
	}
	if( cnt==1 ) {
		ptr[1]=ptr[0];
		ptr[0]=ptr[-1];
		ptr[-1]='0';
	}
}


/* Find ifindex numbers of BPN numbers in the table */
int find_ifindex_numbers(char *host)
{
	MYSQL mysql;
	MYSQL_RES *mysqlresult=NULL;
	MYSQL_ROW mysqlrow;
	int num_rows, i;
	struct struct_snmpresult *snmpresult;
	char query[MAXQUERYLEN], buf[100];
	char ifindex_community[100], ifindex_oid[100];
	char ifindex[MAX_IFINDEX_LEN];
	char *ifindex_start;

	strncpy(buf, host, sizeof(buf)-1);
	convert_host(buf);
	
	if((mysql_thread_init() != 0)) {
		syslog(LOG_ERR, "mysql_thread_init failed");
		abort();
	}

	mysql_init(&mysql);

	if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_ERR, "Failed to connect to database. Error: %s\n", mysql_error(&mysql));
		mysql_thread_end();
		return 0;
	}

	sprintf(query, "select bpn, vlan from sw_%s group by 1", buf);
	#ifdef debug
	syslog(LOG_DEBUG, "Query:%s\n", query);
	#endif
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        for(i=0; i<num_rows; i++) {
		    mysqlrow = mysql_fetch_row(mysqlresult);
			/* Append VLAN number into commmunity name: abs17@10 */
			strcpy(ifindex_community, ayarlar->SnmpRO); 
			strcat(ifindex_community, "@");	strcat(ifindex_community, mysqlrow[1]); 
			strcpy(ifindex_oid, IFINDEXOID); strcat(ifindex_oid, "."); strcat(ifindex_oid, mysqlrow[0]);
			#ifdef debug
			syslog(LOG_DEBUG,"VLAN:%s, BPN:%s --> ifindex_oid:%s, community:%s\n", mysqlrow[1], mysqlrow[0], \
				ifindex_oid, ifindex_community);
			#endif

			/* snmpresult->value holds the snmp response values */
			snmpresult=snmpwalk(host, ifindex_community, ifindex_oid);
			if( snmpresult ) {
				ifindex_start=index(snmpresult->value, (char)' ');
				strncpy(ifindex, ifindex_start+1, MAX_IFINDEX_LEN);
				#ifdef debug
				syslog(LOG_DEBUG, "Host:%s, ifindex:%s\n", host, ifindex);
				#endif
				strncpy(buf, host, sizeof(buf)-1);
				convert_host(buf);

				sprintf(query, "update sw_%s set ifindex='%s' where bpn='%s'", buf, ifindex, mysqlrow[0]);
				mysql_query_trackerd(query, ayarlar);
		
				remove_snmpresult(snmpresult); /* free the chain from the start */
			}
		}
	}
    if( mysqlresult ) mysql_free_result(mysqlresult);
	mysql_close(&mysql);
	mysql_thread_end();
	return 1;
}


/* Find ifnames of ifindex numbers in the table */
int find_ifnames(char *host)
{
	MYSQL mysql;
	MYSQL_RES *mysqlresult=NULL;
	MYSQL_ROW mysqlrow;
	int num_rows, i;
	struct struct_snmpresult *snmpresult;
	char query[MAXQUERYLEN], buf[100];
	char ifname_community[100], ifname_oid[100];
	char ifname[MAX_IFINDEX_LEN];
	char *ifname_start;

	strncpy(buf, host, sizeof(buf)-1);
	convert_host(buf);

	if((mysql_thread_init() != 0)) {
		syslog(LOG_ERR, "mysql_thread_init failed");
		abort();
	}
	
	mysql_init(&mysql);

	if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT, NULL, 0)) {
    	syslog(LOG_ERR, "Failed to connect to database. Error: %s\n", mysql_error(&mysql));
		mysql_thread_end();
		return 0;
	}

	sprintf(query, "select distinct ifindex from sw_%s", buf);
	#ifdef debug
	syslog(LOG_DEBUG, "Query:%s\n", query);
	#endif
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        syslog(LOG_INFO, "Error:%s, Query=%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        for(i=0; i<num_rows; i++) {
		    mysqlrow = mysql_fetch_row(mysqlresult);
			/* Append VLAN number into commmunity name: abs17@10 */
			strcpy(ifname_community, ayarlar->SnmpRO);
			strcpy(ifname_oid, IFNAMEOID); strcat(ifname_oid, "."); strcat(ifname_oid, mysqlrow[0]);
			#ifdef debug
			syslog(LOG_DEBUG,"IfIndex:%s --> ifindex_oid:%s, community:%s\n", mysqlrow[0], ifname_oid, ifname_community);
			#endif
			snmpresult=snmpwalk(host, ifname_community, ifname_oid);
			/* snmpresult->value holds the snmp response values 
				.1.3.6.1.2.1.31.1.1.1.1.26 2/18 */
			if( snmpresult ) {
				ifname_start=index(snmpresult->value, (char)' ');
				strncpy(ifname, ifname_start+1, MAX_IFINDEX_LEN);
				#ifdef debug
				syslog(LOG_DEBUG,"host:%s, ifname:%s\n", host, ifname);
				#endif
				strncpy(buf, host, sizeof(buf)-1);
				convert_host(buf);

				sprintf(query, "update sw_%s set ifname='%s' where ifindex='%s'", buf, ifname, mysqlrow[0]);
				mysql_query_trackerd(query, ayarlar);
		
				remove_snmpresult(snmpresult); /* free the chain from the start */
			}
		}
	}
    if( mysqlresult ) mysql_free_result(mysqlresult);
	mysql_close(&mysql);
	mysql_thread_end();
	return(1);
}

/* All MACs in the sw_IP tables will be inserted into the macs table */
int mac_table(char **switchler, int switch_sayisi, char *zaman)
{
	MYSQL mysql;
    MYSQL_RES *mysqlresultA=NULL;
    MYSQL_RES *mysqlresultB=NULL;
	MYSQL_ROW mysqlrowA, mysqlrowB;
	int num_rows=0, num_rowsA, num_rowsB, i, ii;
	char switchIP[100], query[MAXQUERYLEN];
	char *host, *oldest_time;

#ifdef TRACKERDTEST
	sprintf(query, "drop table if exists macs");
	mysql_query_trackerd(query, ayarlar);

	sprintf(query, "CREATE TABLE macs ( mac char(17), ifname char(10), swip char(15), ip char(15) default 'NOIP', time char(17), uid char(15) default 'NOUID', ifindex int(11) default -1, vlan int(11) default -1) TYPE=MyISAM");
	mysql_query_trackerd(query, ayarlar);

#endif

	mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT,NULL,0)) {
		syslog(LOG_ERR, "Failed to connect to database. Error: %s\n", mysql_error(&mysql));
		return 0;
	}

	for(i=0; i<switch_sayisi; i++) {
		host=switchler[i];
		strncpy(switchIP, host, sizeof(switchIP)-1);
		convert_host(switchIP);

		sprintf(query, "select mac, ifname, vlan, ifindex from sw_%s where cdp='dis' group by ifname", switchIP);
		#ifdef debug
		syslog(LOG_DEBUG, "Host:%s, Query:%s\n", host, query);
		#endif
		if( mysql_real_query(&mysql, query, strlen(query)) ) {
	    	syslog(LOG_ERR, "Error:%s, Query=%s", mysql_error(&mysql), query);
		}
		else { /* query succeeded, process any data returned by it */
			mysqlresultA = mysql_store_result(&mysql);
			num_rowsA = mysql_num_rows(mysqlresultA);
			for(ii=0; ii<num_rowsA; ii++) {
				mysqlrowA = mysql_fetch_row(mysqlresultA);			
				/* if(	check_duplicate_mac(mysqlrowA[0], zaman) ) {
					syslog(LOG_INFO, "Host:%s, duplicated MAC:%s, Switch:%s, ifname:%s\n", mysqlrowA[0], host, mysqlrowA[1], host);
					continue;
				} */
				/* Count the MAC in the database */
                sprintf(query, "select mac, time from macs where mac='%s' order by time", mysqlrowA[0]);
				#ifdef debug
                syslog(LOG_DEBUG, "Host:%s, query:%s\n", host, query);
				#endif
                if( mysql_real_query(&mysql, query, strlen(query)) ) {
					syslog(LOG_ERR, "Error:%s, Query:%s", mysql_error(&mysql), query);
                    continue;
                }
                else { /* query succeeded, process any data returned by it */
                	mysqlresultB = mysql_store_result(&mysql);
                    num_rowsB = mysql_num_rows(mysqlresultB);
                    syslog(LOG_INFO, "mac=%s is found %d times in the database", mysqlrowA[0], num_rows);
                    /* the MAC is not in the database or found less than 'HistoryCount' times, so, insert the record */
                    if( num_rowsB < ayarlar->HistoryCount ) {
                    	sprintf(query, "insert into macs values ( '%s','%s','%s','NOIP','%s','NOUID','%s','%s')",mysqlrowA[0], mysqlrowA[1], host, zaman, mysqlrowA[2], mysqlrowA[3]);
						num_rows=mysql_query_trackerd(query, ayarlar);
						if( num_rows==-1 ) continue;
                        else { /* insert query applied */
                        	if( num_rows>0 ) { /* insert basarili, geriye bir data gelmez. */
                        		syslog(LOG_INFO, "MAC inserted.\n");
								#ifdef HAVE_LDAP
								/* Finds the uid of the MAC in the LDAP database and updates the tracker database with that value */
								ldap_uid_ekle(mysqlrowA[0], zaman, ayarlar);
								#endif
                        	}
                        	else { /* insert basarisiz */
                        		syslog(LOG_INFO, "MAC couldn't be inserted!\n");
                        	}
                        }
					} /* end of insert query */
                    else { /* There are 'HistoryCount' records in the database */
                    	   /* find the oldest record and update it */
                    	mysqlrowB = mysql_fetch_row(mysqlresultB);
                        oldest_time=mysqlrowB[1];
						/* update the record wich has the oldest time */
                        sprintf(query, "update macs set ifname='%s', swip='%s', ip='NOIP', time='%s', uid='NOUID', vlan='%s', ifindex='%s' where mac='%s' and time='%s'", mysqlrowA[1], host, zaman, mysqlrowA[2], mysqlrowA[3], mysqlrowA[0], oldest_time);
						num_rows=mysql_query_trackerd(query, ayarlar);
						if( num_rows==-1 ) continue;
                        /* update query applied */
                        if( num_rows>0 ) {/* update basarili, geriye bir data gelmez. */
                        	syslog(LOG_INFO, "MAC updated.\n");
							#ifdef HAVE_LDAP
							/* Finds the uid of the MAC in the LDAP database and updates the tracker database with that value */
							ldap_uid_ekle(mysqlrowA[0], zaman, ayarlar);
							#endif
						}
                        else syslog(LOG_INFO, "MAC couldn't be updated.\n");
                    }
					if( mysqlresultB ) mysql_free_result(mysqlresultB);
        		}
			}
			if( mysqlresultA ) mysql_free_result(mysqlresultA);
		}
	}

	mysql_close(&mysql);
	return 1;
}

int check_duplicate_mac(char *mac, char *zaman)
{
	MYSQL mysql;
	MYSQL_RES *mysqlresult;
	MYSQL_ROW mysqlrow;
	int num_rows, i;
	char query[MAXQUERYLEN];

	if((mysql_thread_init() != 0)) {
		syslog(LOG_ERR, "mysql_thread_init failed");
		abort();
	}
	
	mysql_init(&mysql);
    if(!mysql_real_connect(&mysql, ayarlar->sqlhost, ayarlar->sqluser, ayarlar->sqlpass, ayarlar->sqldb, SQLPORT,NULL,0)) {
		syslog(LOG_ERR, "Failed to connect to database. Error: %s\n", mysql_error(&mysql));
		mysql_thread_end();
		return 0;
	}
	
	sprintf(query, "select swip, ifname, IP from macs where mac='%s' and time='%s'", mac, zaman);
	syslog(LOG_INFO, query);
	if( mysql_real_query(&mysql, query, strlen(query)) ) {
		syslog(LOG_ERR, "Error:%s, Query:%s", mysql_error(&mysql), query);
		num_rows=0;
	}
    else { // query succeeded, process any data returned by it
           // DUPLICATE MAC !
		mysqlresult = mysql_store_result(&mysql);
		num_rows = mysql_num_rows(mysqlresult);
		for(i=0; i<num_rows; i++) {
			mysqlrow = mysql_fetch_row(mysqlresult);
			syslog(LOG_NOTICE, "Duplicate MAC found: MAC=%s, SwIP=%s, ifname=%s, IP=%s, Time=%s", mac, mysqlrow[0], mysqlrow[1], mysqlrow[2], zaman);
		}
		if( mysqlresult ) mysql_free_result(mysqlresult);
		if( num_rows ) {
			syslog(LOG_NOTICE, "Duplicate MAC Sayisi:%d", num_rows);
		}
	}
	mysql_close(&mysql);
	mysql_thread_end();
	return num_rows;
}


/* Converts formats "0.11.102.15.104.21" --> hexmac=00:0b:66:0f:68:15 */
void macc(char *str)
{
	int i;
	char mac[6][4];
	char *ptr=str, *p;
	
	for(i=0; i<5; i++) {
		/* Find position of next . and replace with NULL */
		p=index(ptr, '.'); *p=(char)NULL;
		/* Copy the small string into mac array */
		strncpy(mac[i], ptr, 3);
		/* Replace again NULL with */
		*p='.';
		/* Advance the string pointer after */
		ptr=++p;
	}
	/* There isn't . at the end of the string
	so apply the same operation one more time */ 
	strcpy(mac[i], ptr);
	
	/* Write the value in hexadecimal with 2 numbers */
	sprintf(str, "%02x:%02x:%02x:%02x:%02x:%02x", (unsigned)strtol(mac[0], NULL, 10),(unsigned)strtol(mac[1], NULL, 10),(unsigned)strtol(mac[2], NULL, 10),(unsigned)strtol(mac[3], NULL, 10),(unsigned)strtol(mac[4], NULL, 10),(unsigned)strtol(mac[5], NULL, 10)  );
}



/* Find CDP enabled ports  */
int find_cdp(char *host)
{
	struct struct_snmpresult *snmpresult, *start;
	char query[MAXQUERYLEN], buf[100];
	char ifindex[10];
	char *position_lastDot;

	/* Start of the chain */
	start=snmpresult=snmpwalk(host, ayarlar->SnmpRO, CDPOID);
	/* snmpresult->value holds the snmp response values 
		.1.3.6.1.4.1.9.9.23.1.2.1.1.3.9.2 1 --> 2 is the ifindex number */
	while( snmpresult ) {
		position_lastDot=rindex(snmpresult->value, (char) '.');
		*position_lastDot=(char) NULL;
		strncpy(ifindex, snmpresult->value+strlen(CDPOID)+1, 10);
		*position_lastDot=' ';
		strncpy(buf, host, sizeof(buf)-1);
		convert_host(buf);
		sprintf(query, "update sw_%s set cdp='1' where ifindex='%s'", buf, ifindex);
		mysql_query_trackerd(query, ayarlar);
		
		snmpresult=snmpresult->next;
	}

	remove_snmpresult(start); /* free the chain from the start */
	return 1;
}




int arptable(void)
{
	struct struct_snmpresult *snmpresult, *start;
    int i;
    int router_sayisi=0;
    char **routerlar;
	char *position_macstart, *position_ip;
	char query[MAXQUERYLEN], *host, *ptr;
	char mac[20], ip[20];
	int dotcount;

    routerlar=router_listesi_al(&router_sayisi, ayarlar);

    syslog(LOG_NOTICE, "The Total number of routers:%d", router_sayisi);
    for(i=0; i<router_sayisi; i++) syslog(LOG_NOTICE, "router[%d]:%s", i, routerlar[i]);

    for(i=0; i<router_sayisi; i++) {
        host=routerlar[i];
		#ifdef debug
        syslog(LOG_DEBUG, "ARP listesi icin Router:%s, Community=%s, OID=%s", host, ayarlar->SnmpRO, ARPOID);
		#endif
		/* Start of the chain */
		start=snmpresult=snmpwalk(host, ayarlar->SnmpRO, ARPOID);
		/* snmpresult->value holds the snmp response values */
		while( snmpresult ) {
			/* .1.3.6.1.2.1.3.1.1.2.150.1.10.4.5.81 "00 04 23 C9 DD 8C  " */
			position_macstart=index(snmpresult->value, '"')+1;
			snmpresult->value[ strlen(snmpresult->value)-2 ] =(char)NULL;
			strcpy(mac, position_macstart);
			/* Replace ' ' with ':' in mac */
			ptr=mac;
			while( *ptr ) {
				if( *ptr==' ' ) *ptr=':';
				ptr++;
			}
			position_ip=index(snmpresult->value, ' ');
			*position_ip=(char)NULL; /* this is the end of the IP */
			dotcount=4;
			while(dotcount>0) {
				if( *position_ip=='.' ) dotcount--;
				position_ip--;
			}
			strcpy(ip, position_ip+=2);
			sprintf(query, "update macs set ip='%s' where mac='%s' and time='%s'", ip, mac, ayarlar->LastUpdateTime);
			#ifdef debug
			syslog(LOG_DEBUG, "Value:%s, ip:%s, mac:%s-\n", snmpresult->value, ip, mac);
			syslog(LOG_DEBUG, "ARP: %s\n", query);
			#endif
			mysql_query_trackerd(query, ayarlar);

			snmpresult=snmpresult->next;
		}

		remove_snmpresult(start); /* free the chain from the start */
	}

    for(i=0; i<router_sayisi; i++) free(routerlar[i]);
    free(routerlar);
    return 0;
}




struct struct_snmpresult *snmpwalk(char *host, char *mycommunity, char *myoid)
{
	void					*sessp;
	struct snmp_session 	session, *sptr;
    netsnmp_pdu				*pdu, *response;
    netsnmp_variable_list 	*vars;
	int 					liberr, syserr;
	char 					*errstr;
    oid             		name[MAX_OID_LEN];
    size_t          		name_length;
    oid             		root[MAX_OID_LEN];
    size_t          		rootlen;
    int             		count, running=1, status, check=0, exitval = 0,	numprinted=0;
    char 					*buf;
	struct 					struct_snmpresult *snmpresult=NULL;


	/* Initialize a "session" that defines who we're going to talk to */
    snmp_sess_init( &session ); /* set up defaults */
    //session.peername = strdup(host);
    session.peername = host;
    session.version = SNMP_VERSION_2c; /* set the SNMP version number */ 
	
    /* open an SNMP session */
	sessp = snmp_sess_open(&session);
	if (!sessp) {
        /* diagnose snmp_open errors with the input snmp_sess_init pointer  */
		/* Error codes found in open calling argument */
		snmp_sess_error(&session, &liberr, &syserr, &errstr);
		syslog(LOG_ERR, "SNMP create error %s.\n", errstr);
        snmp_sess_perror("snmpwalk", &session);
		free(errstr);
		return 0;
	}
	sptr = snmp_sess_session(sessp); /* <-- get the snmp_session pointer */
    /* Pass sptr to snmp_sess_error from here forward */

	/* Change the community name */
	if( !sptr->community) free(sptr->community);
	sptr->community = (u_char *)strdup(mycommunity);
	sptr->community_len = strlen(mycommunity);
	
	#ifdef debug
	syslog(LOG_DEBUG, "Host:%s, snmpwalk -v %ld -c %s %s %s\n", host, session.version, sptr->community, session.peername, myoid);
	#endif

    /* get the initial object and subtree  */
    rootlen = MAX_OID_LEN;
	read_objid(myoid, root, &rootlen);

    /* get first object to start walk  */
    memmove(name, root, rootlen * sizeof(oid));
    name_length = rootlen;
	

	while(running) {
		/* create PDU for GETNEXT request and add object name to request */
		pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
		snmp_add_null_var(pdu, name, name_length);

		/* do the request */
		status = snmp_sess_synch_response(sessp, pdu, &response);
		if(status == STAT_SUCCESS) {
			if(response->errstat == SNMP_ERR_NOERROR) {
				/*  check resulting variables 	*/
				for(vars = response->variables; vars; vars = vars->next_variable) {
					if((vars->name_length < rootlen) || (memcmp(root, vars->name, rootlen * sizeof(oid)) != 0)) {
						/*  not part of this subtree */
						running = 0;
						continue;
					}
 
					if( (vars->type != SNMP_ENDOFMIBVIEW) && (vars->type != SNMP_NOSUCHOBJECT) && (vars->type != SNMP_NOSUCHINSTANCE)) {
						/*  not an exception value 	*/
						if(check && snmp_oid_compare(name, name_length, vars->name, vars->name_length) >= 0) {
							fprintf(stderr, "host:%s, Error: OID not increasing: ", host);
							fprint_objid(stderr, name, name_length);
							fprintf(stderr, " >= ");
							fprint_objid(stderr, vars->name, vars->name_length);
							fprintf(stderr, "\n");
							running = 0;
							exitval = 1;
						}

						numprinted++; /* Allocates a string memory, prints into it and returns its address */
						buf=fprint_variable_string(vars->name, vars->name_length, vars);
            			if( buf ) {
							#ifdef debug
		    				syslog(LOG_DEBUG,"host:%s, snmp response:%s\n", host, buf);
							#endif
							snmpresult=add_snmpresult(snmpresult);
							snmpresult->value=buf;
            			}
						else {
							syslog(LOG_ERR, "snmpwalk: Couldn't allocate memory for string\n");
							continue;
						}
						
						memmove((char *) name, (char *) vars->name, vars->name_length * sizeof(oid));
						name_length = vars->name_length;
					}
					else /* an exception value, so stop */ {
						running = 0;
					}
				}
			} 
			else {
				/* error in response, print it 	*/
				running = 0;
				if(response->errstat == SNMP_ERR_NOSUCHNAME) {
					#ifdef debug
					syslog(LOG_DEBUG,"host:%s, snmpwalk: End of MIB\n", host);
					#endif
				}
				else {
					syslog(LOG_INFO, "host:%s, Error in packet.\nReason: %s\n",host, snmp_errstring(response->errstat));
					if(response->errindex != 0) {
						syslog(LOG_INFO, "Host:%s, Failed object: ", host);
						for(count = 1, vars = response->variables; vars && count != response->errindex; vars = vars->next_variable, count++) /*EMPTY*/;
						if(vars) fprint_objid(stderr, vars->name, vars->name_length);
						fprintf(stderr, "\n");
					}
					exitval = 2;
				}
			}
		}
		else {
			if (status == STAT_TIMEOUT) {
				syslog(LOG_NOTICE, "Host:%s, Timeout: No Response\n", session.peername);
				running = 0;
				exitval = 1;
			}
			else { /* status == STAT_ERROR */
				syslog(LOG_NOTICE, "Host:%s, snmpwalk status=error\n", host);
				running = 0;
				exitval = 1;
			}
		}
		if(response) snmp_free_pdu(response);
	}

	if(numprinted == 0 && status == STAT_SUCCESS) {
		/*
		* no printed successful results, which may mean we were
		* pointed at an only existing instance.  Attempt a GET, just
		* for get measure. 
		*/
		snmpresult=snmp_get_and_print_tracker(host, sessp, root, rootlen, snmpresult);
	}

	if( !sptr->community) free(sptr->community);

	snmp_sess_close(sessp);
	/* If there isn't any response return NULL */
	if(!snmpresult) {
		return NULL;
	}
	/* Return start of the chain */
	return snmpresult->start;
}

struct struct_snmpresult *snmp_get_and_print_tracker(char *host, void *sessp, oid *theoid, size_t theoid_len, struct struct_snmpresult *snmpresult)
{
    netsnmp_pdu *pdu, *response;
    netsnmp_variable_list *vars;
    int  status, count;
	char *buf;

    pdu = snmp_pdu_create(SNMP_MSG_GET);
    snmp_add_null_var(pdu, theoid, theoid_len);

	status = snmp_sess_synch_response(sessp, pdu, &response);
	if(status == STAT_SUCCESS) {
		if(response->errstat == SNMP_ERR_NOERROR) {
			/*  check resulting variables 	*/
			for(vars = response->variables; vars; vars = vars->next_variable) {
				if( (vars->type != SNMP_ENDOFMIBVIEW) && (vars->type != SNMP_NOSUCHOBJECT) && (vars->type != SNMP_NOSUCHINSTANCE)) {
					/*  not an exception value 	*/
					/* Allocates a string memory, prints into it and returns its address */
					buf=fprint_variable_string(vars->name, vars->name_length, vars);
           			if( buf ) {
						#ifdef debug
	    				syslog(LOG_DEBUG,"host:%s, snmp result:%s\n", host, buf);
						#endif
						snmpresult=add_snmpresult(snmpresult);
						snmpresult->value=buf;
           			}
					else {
						syslog(LOG_ERR, "Couldn't allocate memory for string.\n");
						continue;
					}
				}
				else { /* an exception value, so stop */ 
					syslog(LOG_INFO, "host:%s, snmp_get_and_print_tracker, exception in snmpwalk query, return without a vlaue.\n", host);
					if(response) snmp_free_pdu(response);
					return snmpresult;
				}
			}
		} 
		else {
			/* error in response, print it 	*/
			syslog(LOG_INFO,"host:%s, error in snmp response !\n", host);
			if(response->errstat == SNMP_ERR_NOSUCHNAME) {
				syslog(LOG_INFO,"host:%s, End of MIB\n", host);
			}
			else {
				syslog(LOG_ERR, "Host:%s, Error in packet.\nReason: %s\n",host, snmp_errstring(response->errstat));
				if(response->errindex != 0) {
					syslog(LOG_ERR, "Host:%s, Failed object: ", host);
					for(count = 1, vars = response->variables; vars && count != response->errindex; vars = vars->next_variable, count++) /*EMPTY*/;
					if(vars) fprint_objid(stderr, vars->name, vars->name_length);
					syslog(LOG_INFO, "\n");
				}
				
			}
		}
	}
	else {
		if (status == STAT_TIMEOUT) {
			syslog(LOG_INFO, "Host:%s, Timeout: No Response\n", host);
		}
		else { /* status == STAT_ERROR */
			syslog(LOG_ERR, "Host:%s, snmp_get_and_print_tracker: status=error\n", host);
			snmp_sess_perror("snmpwalk", sessp);
		}
	}
	if(response) snmp_free_pdu(response);
	return snmpresult;
}


char *fprint_variable_string(const oid * objid, size_t objidlen, const netsnmp_variable_list * variable)
{
   u_char *buf = NULL;
   u_char *string=NULL;
   size_t buf_len = 256, out_len = 0;

	if( !(string=(u_char *) calloc(buf_len+1, 1) ) ) {
		return NULL;
	}

	if ((buf = (u_char *) calloc(buf_len, 1)) == NULL) {
		return NULL;
	} else {
		if (sprint_realloc_variable(&buf, &buf_len, &out_len, 1, objid, objidlen, variable)) {
			sprintf((char *)string, "%s", buf);
		} else {
			SNMP_FREE(string); string=NULL;
		}
	}
   return (char *)string;
}
